using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.IO;
using System.Drawing.Printing;

using Screentaker.Helper;

namespace Screentaker
{
    public partial class FormCaptured : Form
    {
        public FormCaptured()
        {
            InitializeComponent();
        }

        #region Internals

        private bool _CloseOnAction              = STSystem.Settings.GetBool("CloseOnAction");
        private FormColorPicker _ColorPicker    = new FormColorPicker();

        #endregion

        /// <summary>
        /// Zeigt das Fenster an und fgt den Screenshot in das Fenster ein. Weiterhin wird das 
        /// Fenster automatisch an die Gre des Screenshots angepasst
        /// </summary>
        /// <param name="Screenshot"></param>
        public void Show(Image Screenshot)
        {
            if (Screenshot != null)
            {
                this.pBoxShot.Image = Screenshot;
                ResizeToScreenshot();
            }

            #region Alle vorhandenen Drucker ermitteln

            try
            {
                foreach (string _CurrentPrintername in PrinterSettings.InstalledPrinters)
                {

                    ButtonPrint.DropDownItems.Add(_CurrentPrintername);
                }
            }
            catch { /*Nichts tun, kein Drucker vorhanden!*/ }

            #endregion

            #region Gre des Fensters anpassen

            //Gre des Fensters anpassen falls das angezeigt 
            //Bild grer als der Arbeitsbereich ist
            if ((this.Width > Screen.PrimaryScreen.WorkingArea.Width) || (this.Height > Screen.PrimaryScreen.WorkingArea.Height))
            {
                this.WindowState = FormWindowState.Maximized;
            }

            //Falls konfiguriert wird dieses Fenster immer maximized angezeigt
            if (STSystem.Settings.GetBool("AlwaysOpenMaximized") == true)
            {
                this.WindowState = FormWindowState.Maximized;
            }

            #endregion

            #region Falls konfiguriert Dateinamen setzen


            if (STSystem.Settings.GetBool("AutoNumerate"))
            {
                //Zahl hinter den Standard-Dateinamen hinzufgen
                TextBoxFilename.Text = STSystem.GetUnusedFilename(STSystem.Settings.GetString("HomeDir") +
                                        @"\", STSystem.Settings.GetString("DefaultFilename"),
                                        string.Empty);
                                        
            }
            else
            {
                //Nur Dateinamen setzen
                TextBoxFilename.Text = STSystem.Settings.GetString("DefaultFilename");
            }

            #endregion

            UpdateStatusbar();

            //Externen Toolnamen einsetzen
            ButtonSendToTool.ToolTipText = "Mit externem Programm bearbeiten\r\n\r\n(" + STSystem.Settings.GetString("ExternalToolName") + ")";

            pBoxShot.InnerControl.MouseMove += new MouseEventHandler(ImageBox_MouseMove);

            this.Show();
        }

        /// <summary>
        /// Passt das Fenster dem Screenshot an
        /// </summary>
        private void ResizeToScreenshot()
        {
            this.Height = pBoxShot.Image.Height + toolStripMain.Height + statusStripMain.Height + 60;
            this.Width = pBoxShot.Image.Width + 30;
        }

        /// <summary>
        /// Wenn die Maus ber das Image-Control der pBox-Shot bewegt wird
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ImageBox_MouseMove(object sender, MouseEventArgs e)
        {
            UpdateStatusbar();
        }

        #region Screenshot speichern

        /// <summary>
        /// Image als JPG-Sichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSaveJPG_Click(object sender, EventArgs e)
        {
            SaveImage(ImageProcessing.EncoderByDesc.JPG, ".jpg");
        }

        /// <summary>
        /// Bild als PNG sichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSaveAsPNG_Click(object sender, EventArgs e)
        {
            SaveImage(ImageProcessing.EncoderByDesc.PNG, ".png");
        }

        /// <summary>
        /// Bild als BMP Speichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSaveAsBMP_Click(object sender, EventArgs e)
        {
            SaveImage(ImageProcessing.EncoderByDesc.BMP, ".bmp");
        }

        /// <summary>
        /// Bild als TIFF speichern
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSaveAsTIFF_Click(object sender, EventArgs e)
        {
            SaveImage(ImageProcessing.EncoderByDesc.TIFF, ".gif");
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="Format"></param>
        /// <param name="Extension"></param>
        private void SaveImage(ImageProcessing.EncoderByDesc EncDesc, string Extension)
        {
            if (TextBoxFilename.Text == string.Empty)
            {
                Messages.WarningBox(this, "Bitte geben Sie einen Dateinamen ein.\r\nDer Vorgang wird abgebrochen.");
                return;
            }

            //Verzeichnis ermitteln und falls ntig anlegen
            string _Target = STSystem.Settings.GetString("HomeDir");
            if (!Directory.Exists(_Target)) { Directory.CreateDirectory(_Target); }

            _Target += @"\" + TextBoxFilename.Text + Extension;

            //Falls die Datei bereits existiert nachfragen
            if (File.Exists(_Target) == true)
            {
                if (MessageBox.Show(this, "Die Datei \"" + _Target + "\" existiert bereits. Mchten Sie die vorhandene Datei berschreiben?",
                    STSystem.AppTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
                {
                    return;
                }
            }

            ImageProcessing.CompressImage(pBoxShot.Image, EncDesc, _Target);

            //Schlieen falls konfiguriert
            if (_CloseOnAction == true)
            {
                this.Close();
            }
        }

        #endregion

        /// <summary>
        /// ffnet den Screenshot zum Bearbeiten in einem Externen Tool, welches vorher
        /// definiert wurde
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSendToTool_Click(object sender, EventArgs e)
        {
            //Das Fenster wird ausgeblendet, das externe Tool gestartet und nach dem
            //Bearbeiten wird das Fenster wieder geffnet und das neue Image angezeigt
            this.Hide();
            string AppFile = STSystem.Settings.GetString("ExternalToolFile");
            string AppName = STSystem.Settings.GetString("ExternalToolName");

            pBoxShot.Image = ImageProcessing.EditExternal(pBoxShot.Image, AppFile, AppName, true);
            
            AppFile = null;
            AppName = null;

            //Bearbeitetes Bild der History hinzufgen
            if (STSystem.Settings.GetBool("HistoryTakeEdited"))
            {
                History.Add(pBoxShot.Image);
            }

            if (STSystem.Settings.GetBool("NoLoadAfterEdit"))
            { this.Close(); }
            else { this.Show(); }
        }

        /// <summary>
        /// Aktualisiert die Statusbar
        /// </summary>
        private void UpdateStatusbar()
        {
            if (pBoxShot.Image == null)
            {
                LabelImageInf.Text = "Keine Informationen verfgbar!";
                return;
            }

            try
            {
                //Informationen in der Statusleiste anzeigen
                LabelImageInf.Text = string.Empty;

                if (pBoxShot.IsZoomed)
                {
                    LabelImageInf.Text      = "Informationen stehen im Zoom-Modus nicht zur Verfgung!";
                }
                else
                {
                    LabelImageInf.Text += "Hhe: " + pBoxShot.Image.Height.ToString() + "px ";
                    LabelImageInf.Text += "Breite: " + pBoxShot.Image.Width.ToString() + "px ";

                    LabelImageInf.Text += "| ";

                    int _CurrentX = pBoxShot.CursorPosition.X;
                    int _CurrentY = pBoxShot.CursorPosition.Y;

                    LabelImageInf.Text += "X: " + Generators.FillNumber(_CurrentX, pBoxShot.Image.Width) + " ";
                    LabelImageInf.Text += "Y: " + Generators.FillNumber(_CurrentY, pBoxShot.Image.Height) + " ";
                }
            }
            catch
            {
                LabelImageInf.Text = "Imagefehler! Die Bilddatei ist ungltig.";
            }
        }

        /// <summary>
        /// Speichert den Screenshot unter Verwendung des Speichern-Unter-Dialogs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSaveAs_Click(object sender, EventArgs e)
        {
            if (pBoxShot.Image == null)
            {
                MessageBox.Show("Es ist kein zu speicherndes Bild vorhanden!", "Warnung", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            using (SaveFileDialog _SaveDialog = new SaveFileDialog())
            {
                _SaveDialog.AddExtension = true;
                _SaveDialog.RestoreDirectory = true;
                _SaveDialog.Filter = "JPEG (*.jpg)|*.jpg|PNG(*.png)|*.png|Bitmap (*.bmp)|*.bmp|GIF (*.gif)|*.gif";

                if (_SaveDialog.ShowDialog(this) == DialogResult.OK || ! _SaveDialog.FileName.Equals(string.Empty))
                {
                    ImageProcessing.CompressImage(pBoxShot.Image,
                        ImageProcessing.GetEncDescription(new FileInfo(_SaveDialog.FileName).Extension),
                        _SaveDialog.FileName);

                    //Fenster schlieen falls gewnscht
                    if (_CloseOnAction == true)
                    {
                        this.Close();
                    }
                }
            }
        }

        /// <summary>
        /// Kopiert den aktuellen Screenshot in die Zwischenablage
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClipboard_Click(object sender, EventArgs e)
        {
            Clipboard.SetImage(pBoxShot.Image);

            if (_CloseOnAction == true)
            {
                this.Close();
            }
        }

        /// <summary>
        /// ffnet das Einstellungen-Fenster
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonConfig_Click(object sender, EventArgs e)
        {
            using (FormConfig _Configuration = new FormConfig())
            {
                _Configuration.StartPosition    = FormStartPosition.CenterParent;
                _Configuration.ShowInTaskbar    = false;
                _Configuration.ShowDialog(this);
            }
        }

        /// <summary>
        /// Zeigt das "ber - Fenster" an
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemAbout_Click(object sender, EventArgs e)
        {
            using (FormAbout _About = new FormAbout())
            {
                _About.ShowInTaskbar    = false;
                _About.ShowInCorner     = false;
                _About.ShowDialog(this);
            }
        }

        /// <summary>
        /// Zeigt das History-Fenster an
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonHistory_Click(object sender, EventArgs e)
        {
            using (FormHistory _History = new FormHistory())
            {
                _History.ShowInTaskbar = false;

                if (_History.ShowDialog(this) == DialogResult.OK)
                {
                    pBoxShot.Image = _History.SelectedImage;
                }
            }
        }

        /// <summary>
        /// Laden des Fensters
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormCaptured_Load(object sender, EventArgs e)
        {
            STSystem.AnimateWindow(this);
        }

        /// <summary>
        /// Ldt das Image via Imageshack.us hoch
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemImageshack_Click(object sender, EventArgs e)
        {
            UploadTo(Uploader.UploadType.Imageshack);
        }

        /// <summary>
        /// Ldt das Image via DirectUpload hoch
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemDirectUpload_Click(object sender, EventArgs e)
        {
            UploadTo(Uploader.UploadType.DirectUpload);
        }

        /// <summary>
        /// Ldt einen Screenshot an einen bestimmten Provider hoch
        /// </summary>
        /// <param name="Provider"></param>
        private void UploadTo(Uploader.UploadType Provider)
        {
            using (FormUploadResult _UploadView = new FormUploadResult())
            {
                _UploadView.Screenshot  = pBoxShot.Image;
                _UploadView.Provider    = Provider;

                _UploadView.ShowDialog(this);
            }
        }

        /// <summary>
        /// Setzt die Stati aller Buttons fr das erweiterte bearbeiten zurck
        /// </summary>
        private void ResetEditModes()
        {
            ButtonDrawLine.Checked = false;
            ButtonDrawRect.Checked = false;
            ButtonDrawString.Checked = false;

            pBoxShot.SetMode(PictureBoxEx.EditModes.None);
        }

        #region Hotkeys fr Funktionen dieses Fensters

        /// <summary>
        /// Hotkeys die nur fr dieses Fenster gelten
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormCaptured_KeyDown(object sender, KeyEventArgs e)
        {
            Keys k = e.KeyData;

            if (k == (Keys.Control | Keys.S))
            {
               ButtonSaveAs_Click(null, null);
            }
            else if (k == (Keys.Control | Keys.E))
            {
                ButtonSendToTool_Click(null, null);
            }
            else if (k == (Keys.Control | Keys.C))
            {
                this.Close();
            }
        }

        #endregion

        /// <summary>
        /// ffnet den Auswahlbereich um einen Screenshot erneut beschneiden zu knnen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonReCapture_Click(object sender, EventArgs e)
        {
            this.BackColor = Color.White;
            new ImageGrabbing().Grab(ImageGrabbing.GrabType.Selection);
            this.Close();
        }

        /// <summary>
        /// Zoomt den Screenshot um 1 Faktor
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonZoomIn_Click(object sender, EventArgs e)
        {
            pBoxShot.ZoomIn();
        }

        /// <summary>
        /// Zoomt den Screenshot um 1 Faktor kleiner
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonZoomOut_Click(object sender, EventArgs e)
        {
            pBoxShot.ZoomOut();
        }

        /// <summary>
        /// Setzt den Screenshot wieder auf die Standardgre zurck und 
        /// zentriert ihn
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonOriginalSize_Click(object sender, EventArgs e)
        {
            pBoxShot.SetOriginalSize();
            ResizeToScreenshot();
        }

        /// <summary>
        /// ffnet die URL die die Hilfe bereitstellt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemDoku_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(STSystem.HelpRequestIDs.editing);
        }

        /// <summary>
        /// Fenster ffnen um die Qualittseinstellungen anzupassen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonQuality_Click(object sender, EventArgs e)
        {
            using (FormQualityWizard _QWizard = new FormQualityWizard())
            {
                _QWizard.pictureBoxExOriginal.Image = pBoxShot.Image;

                if (_QWizard.ShowDialog(this) == DialogResult.OK)
                {
                    pBoxShot.Image = _QWizard.pictureBoxExSized.Image;
                   
                    //Falls gewnscht in die History bernehmen
                    if (STSystem.Settings.GetBool("HistoryTakeSized"))
                    {
                        History.Add(_QWizard.pictureBoxExSized.Image);
                    }
                }
            }
        }

        /// <summary>
        /// Rft das Informationsfenster zu diesem Programm auf 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAbout_Click(object sender, EventArgs e)
        {
            using (FormAbout _About = new FormAbout())
            {
                _About.ShowInCorner = false;
                _About.ShowDialog(this);
            }
        }

        #region Hilfebehandlungen

        /// <summary>
        /// Ruft die Hilfe zu diesem Fenster auf
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonHelp_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.editing);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="hlpevent"></param>
        private void FormCaptured_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.editing);
        }

        #endregion

        /// <summary>
        /// Falls ein Drucker angeklickt wurde wird nun der Printdialog 
        /// mit diesem Drucker angezeigt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonPrint_DropDownItemClicked(object sender, ToolStripItemClickedEventArgs e)
        {
            new Printing().PrintPreview(pBoxShot.Image, e.ClickedItem.Text);
        }

        /// <summary>
        /// Beim schlieen dieses Fensters Resourcen freigeben
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormCaptured_FormClosing(object sender, FormClosingEventArgs e)
        {
            this._ColorPicker.IsParented = false;
            this.pBoxShot.Image.Dispose();
            this.pBoxShot.Image = null;
            this.pBoxShot.Dispose();
            this.pBoxShot = null;
            this.toolStripMain.Dispose();
            this.toolStripMain = null;

            this.Dispose(true);
            GC.Collect();
        }

        #region Bearbeitungsmodi

        /// <summary>
        /// Modus zum zeichnen einer roten Linie aktivieren/deaktivieren
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDrawLine_Click(object sender, EventArgs e)
        {
            if (pBoxShot.EditMode != PictureBoxEx.EditModes.Line)
            {
                ResetEditModes();
                pBoxShot.SetMode(PictureBoxEx.EditModes.Line);
                ButtonDrawLine.Checked = true;
            }
            else
            {
                if (pBoxShot.ImageWasChanged) { History.Add(pBoxShot.Image); }
                ResetEditModes();
            }
        }

        /// <summary>
        /// Modus zum einfgen eines Strings aktivieren/deaktivieren
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDrawString_Click(object sender, EventArgs e)
        {
            if (pBoxShot.EditMode != PictureBoxEx.EditModes.StringInsert)
            {
                pBoxShot.SetMode(PictureBoxEx.EditModes.StringInsert);
            }
            else
            {
                pBoxShot.SetMode(PictureBoxEx.EditModes.None);
            }
        }

        /// <summary>
        /// Stellt das Originalimage wieder her
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRestoreOriginal_Click(object sender, EventArgs e)
        {
            ResetEditModes();
            pBoxShot.RestoreOriginalImage();

            ButtonDrawLine.Checked = false;
            ButtonDrawString.Checked = false;
        }

        /// <summary>
        /// Starten des Colorpicker
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonColorPicker_Click(object sender, EventArgs e)
        {
            _ColorPicker.SourcePictureBox = pBoxShot;
            _ColorPicker.Show();
        }

        /// <summary>
        /// Setzen des Rectangle Tools
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonDrawRect_Click(object sender, EventArgs e)
        {
            if (pBoxShot.EditMode != PictureBoxEx.EditModes.RectangleTool)
            {
                ResetEditModes();
                pBoxShot.SetMode(PictureBoxEx.EditModes.RectangleTool);
                ButtonDrawRect.Checked = true;
            }
            else
            {
                if (pBoxShot.ImageWasChanged) { History.Add(pBoxShot.Image); }
                ResetEditModes();
            }
        }

        #endregion
    }
}