using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using Screentaker.Helper;

namespace Screentaker
{
    public partial class FormColorPicker : Form
    {
        #region Constructor

        public FormColorPicker()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Bewegen der Maus im Quellobjekt
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void InnerControl_MouseMove(object sender, MouseEventArgs e)
        {
            Color _CurrCol = _SourcePictureBox.GetCurrentColor();
            labelCurrentColor.BackColor = _CurrCol;

            textBoxRGB.Text = Generators.FillNumber(_CurrCol.R, 255) +
                    "," + Generators.FillNumber(_CurrCol.G, 255) +
                    "," + Generators.FillNumber(_CurrCol.B, 255);

            textBoxHEX.Text = string.Format("{0:X2}", _CurrCol.R) +
                    string.Format("{0:X2}", _CurrCol.G) +
                    string.Format("{0:X2}", _CurrCol.B);
        }

        /// <summary>
        /// Hinzufgen einer weiteren Farbe
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void InnerControl_MouseClick(object sender, MouseEventArgs e)
        {
            ListViewItem _NewColor = new ListViewItem();
            _NewColor.Text = textBoxRGB.Text;
            _NewColor.SubItems.Add(textBoxHEX.Text);
            _NewColor.BackColor = labelCurrentColor.BackColor;

            listViewColors.Items.Insert(0, _NewColor);
        }

        #endregion

        #region Internals
        
        public PictureBoxEx _SourcePictureBox = new PictureBoxEx();
        public bool IsParented = true;
       
        #endregion

        #region Properties

        /// <summary>
        /// Die Dazugehrige PictureBoxEs, aus der die Informationen ausgelesen werden
        /// sollen.
        /// </summary>
        public PictureBoxEx SourcePictureBox
        {
            get
            {
                return _SourcePictureBox;
            }

            set
            {
                _SourcePictureBox = value;
                _SourcePictureBox.SetMode(PictureBoxEx.EditModes.ColorPicker);
                _SourcePictureBox.InnerControl.MouseMove += new MouseEventHandler(InnerControl_MouseMove);
                _SourcePictureBox.InnerControl.MouseClick += new MouseEventHandler(InnerControl_MouseClick);       
            }
        }

        #endregion

        /// <summary>
        /// Schlieen dieses Fensters
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClose_Click(object sender, EventArgs e)
        {
            if (! IsParented)
            {
                _SourcePictureBox.MouseMove -= new MouseEventHandler(InnerControl_MouseMove);
                _SourcePictureBox.MouseClick -= new MouseEventHandler(InnerControl_MouseClick);
                _SourcePictureBox.Dispose();

                this.Close();
            }
            else
            {
                _SourcePictureBox.SetMode(PictureBoxEx.EditModes.None);
                this.Hide();
            }
        }

        /// <summary>
        /// Falls aus der Liste eine Farbe ausgewhlt wurde
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listViewColors_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listViewColors.SelectedItems.Count == 0)
            {
                return;
            }

            labelCurrentColor.BackColor = listViewColors.SelectedItems[0].BackColor;
            textBoxRGB.Text = listViewColors.SelectedItems[0].SubItems[0].Text;
            textBoxHEX.Text = listViewColors.SelectedItems[0].SubItems[1].Text;
        }

        /// <summary>
        /// Lschen aller Farben
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClear_Click(object sender, EventArgs e)
        {
            listViewColors.Items.Clear();
        }
    }
}