using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Screentaker.Helper;

namespace Screentaker
{
    public partial class FormConfig : Form
    {
        #region Constructor

        public FormConfig()
        {
            InitializeComponent();
        }

        #endregion

        private HotkeyHandling.HOTKEY _HotkeyCompScreenBuffer = new HotkeyHandling.HOTKEY();
        private HotkeyHandling.HOTKEY _HotkeyCurrScreenBuffer = new HotkeyHandling.HOTKEY();
        private HotkeyHandling.HOTKEY _HotkeyDefScreenBuffer = new HotkeyHandling.HOTKEY();
        private HotkeyHandling.HOTKEY _HotkeyUploadBuffer = new HotkeyHandling.HOTKEY();

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Close();         
        }

        /// <summary>
        /// Beim Laden des Fensters werden die Einstellungen eingelesen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormConfig_Load(object sender, EventArgs e)
        {
            if (!STSystem.InternalSettings.GetBool("UseNoUserprofiles"))
            {
                labelHeadline.Text = "Einstellungen anpassen fr " + STSystem.Settings.CurrentUser;
            }
            else
            {
                labelHeadline.Text = "Einstellungen anpassen";
            }

            checkBoxUseNoUserprofiles.Checked = STSystem.InternalSettings.GetBool("UseNoUserprofiles");
            textBoxExternalToolName.Text = STSystem.Settings.GetString("ExternalToolName");
            textBoxExternalToolFile.Text = STSystem.Settings.GetString("ExternalToolFile");
            textBoxHomeDir.Text = STSystem.Settings.GetString("Homedir");
            checkBoxCloseOnAction.Checked = STSystem.Settings.GetBool("CloseOnAction");
            numericUpDownHistoryLenght.Value = STSystem.Settings.GetInt("HistoryLenght");
            checkBoxEnableAutoUpdate.Checked = STSystem.Settings.GetBool("autoupdate");
            checkBoxAutorun.Checked = new OSIntegration().AutorunExists(STSystem.AppTitle);

            _HotkeyCompScreenBuffer = HotkeyHandling.Generate(STSystem.Settings.GetString("hotkeycompscreen"));
            _HotkeyCurrScreenBuffer = HotkeyHandling.Generate(STSystem.Settings.GetString("hotkeycurrscreen"));
            _HotkeyDefScreenBuffer = HotkeyHandling.Generate(STSystem.Settings.GetString("hotkeydefscreen"));
            _HotkeyUploadBuffer = HotkeyHandling.Generate(STSystem.Settings.GetString("HotkeyUpload"));

            checkBoxAlwaysOpenMaximized.Checked = STSystem.Settings.GetBool("AlwaysOpenMaximized");
            checkBoxCloseOnURLLink.Checked = STSystem.Settings.GetBool("CloseOnURLLink");
            checkBoxAutoShowScreen.Checked = STSystem.Settings.GetBool("AutoShowScreen");
            checkBoxHistoryTakeEdited.Checked = STSystem.Settings.GetBool("HistoryTakeEdited");
            checkBoxHistoryTakeSized.Checked = STSystem.Settings.GetBool("HistoryTakeSized");
            checkBoxAutoNumerate.Checked = STSystem.Settings.GetBool("AutoNumerate");
            textBoxDefaultFilename.Text = STSystem.Settings.GetString("DefaultFilename");
            checkBoxAutoSaveScreen.Checked = STSystem.Settings.GetBool("AutoSaveScreen");
            comboBoxAutosaveFiletype.Text = STSystem.Settings.GetString("AutoCompressFiletype");
            numericUpDownQualityAutocompress.Value = new decimal(STSystem.Settings.GetInt("QualityAutocompress"));
            checkBoxNoLoadAfterEdit.Checked = STSystem.Settings.GetBool("NoLoadAfterEdit");
            comboBoxUploadFormat.Text = STSystem.Settings.GetString("UploadFormat");
            checkBoxEnableHistory.Checked = STSystem.Settings.GetBool("HistoryEnabled");
            checkBoxUseTraymenuIcons.Checked = STSystem.Settings.GetBool("UseTraymenuIcons");
            comboBoxHotkeyUploadProvider.Text = STSystem.Settings.GetString("HotkeyUploadProvider");

            labelHKFullScreenValue.Text = FormHotkeyGrabber.GetHKValue(_HotkeyCompScreenBuffer);
            labelHKCurrentScreenValue.Text = FormHotkeyGrabber.GetHKValue(_HotkeyCurrScreenBuffer);
            labelHKDefScreenValue.Text = FormHotkeyGrabber.GetHKValue(_HotkeyDefScreenBuffer);
            labelHKUploadValue.Text = FormHotkeyGrabber.GetHKValue(_HotkeyUploadBuffer);

            checkBoxEnableHistory_CheckedChanged(null, null);
            checkBoxAutoSaveScreen_CheckedChanged(null, null);

            STSystem.AnimateWindow(this);
        }

        /// <summary>
        /// ffnet einen Dialog mit dem das HomeDir gesetzt werden kann
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSetHomeDir_Click(object sender, EventArgs e)
        {
            using (FolderBrowserDialog _Folder = new FolderBrowserDialog())
            {
                _Folder.Description = "Bitte whlen Sie einen Pfad aus in dem Ihre Screenshots gespeichert werden.";

                if (_Folder.ShowDialog() == DialogResult.OK)
                {
                    textBoxHomeDir.Text = _Folder.SelectedPath;
                }
            }
        }

        /// <summary>
        /// Speichern der Einstellungen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSave_Click(object sender, EventArgs e)
        {
            this.Hide();
            
            STSystem.InternalSettings.SetValue("UseNoUserprofiles", checkBoxUseNoUserprofiles.Checked);
            
            STSystem.Settings.SetValue("ExternalToolFile", textBoxExternalToolFile.Text);
            STSystem.Settings.SetValue("ExternalToolName", textBoxExternalToolName.Text);
            STSystem.Settings.SetValue("HistoryEnabled", checkBoxEnableHistory.Checked);
            STSystem.Settings.SetValue("HistoryLenght", numericUpDownHistoryLenght.Value);
            STSystem.Settings.SetValue("Homedir", textBoxHomeDir.Text);
            STSystem.Settings.SetValue("CloseOnAction", checkBoxCloseOnAction.Checked);
            
            STSystem.Settings.SetValue("hotkeycompscreen", HotkeyHandling.Generate(_HotkeyCompScreenBuffer));
            STSystem.Settings.SetValue("hotkeycurrscreen", HotkeyHandling.Generate(_HotkeyCurrScreenBuffer));
            STSystem.Settings.SetValue("hotkeydefscreen", HotkeyHandling.Generate(_HotkeyDefScreenBuffer));
            STSystem.Settings.SetValue("HotkeyUpload", HotkeyHandling.Generate(_HotkeyUploadBuffer));

            STSystem.Settings.SetValue("AlwaysOpenMaximized", checkBoxAlwaysOpenMaximized.Checked);
            STSystem.Settings.SetValue("CloseOnURLLink", checkBoxCloseOnURLLink.Checked);
            STSystem.Settings.SetValue("AutoShowScreen", checkBoxAutoShowScreen.Checked);
            STSystem.Settings.SetValue("HistoryTakeEdited", checkBoxHistoryTakeEdited.Checked);
            STSystem.Settings.SetValue("HistoryTakeSized", checkBoxHistoryTakeSized.Checked);
            STSystem.Settings.SetValue("AutoNumerate", checkBoxAutoNumerate.Checked);
            STSystem.Settings.SetValue("DefaultFilename", textBoxDefaultFilename.Text);
            STSystem.Settings.SetValue("AutoSaveScreen", checkBoxAutoSaveScreen.Checked);
            STSystem.Settings.SetValue("AutoUpdate", checkBoxEnableAutoUpdate.Checked);
            STSystem.Settings.SetValue("AutoCompressFiletype", comboBoxAutosaveFiletype.Text);
            STSystem.Settings.SetValue("QualityAutocompress", numericUpDownQualityAutocompress.Value);
            STSystem.Settings.SetValue("NoLoadAfterEdit", checkBoxNoLoadAfterEdit.Checked);
            STSystem.Settings.SetValue("UploadFormat", comboBoxUploadFormat.Text);
            STSystem.Settings.SetValue("UseTraymenuIcons", checkBoxUseTraymenuIcons.Checked);
            STSystem.Settings.SetValue("HotkeyUploadProvider", comboBoxHotkeyUploadProvider.Text);

            STSystem.Settings.Save();
            STSystem.InternalSettings.Save();

            #region HomeDir anlegen falls nicht vorhanden

            if (!Directory.Exists(textBoxHomeDir.Text))
            {
                Directory.CreateDirectory(textBoxHomeDir.Text);
            }

            #endregion

            #region Hotkeys aktualisieren

            HotkeyHandling.UnregisterAll(STSystem.STController);
            STSystem.CurrentHKCurr = _HotkeyCurrScreenBuffer;
            STSystem.CurrentHKFull = _HotkeyCompScreenBuffer;
            STSystem.CurrentHKDef = _HotkeyDefScreenBuffer;
            STSystem.CurrentHKUpload = _HotkeyUploadBuffer;

            STSystem.STController.RegisterHotkeys();

            #endregion

            #region Autostart konfigurieren

            if (checkBoxAutorun.Checked == true)
            {
                new OSIntegration().CreateAutorun(STSystem.AppTitle, Application.ExecutablePath);
            }
            else
            {
                new OSIntegration().DeleteAutorun(STSystem.AppTitle);
            }

            #endregion

            #region History aktualisieren

            if (checkBoxEnableHistory.Checked == false)
            {
                History.Lenght = 0;
            }
            else
            {
                if (History.Lenght != (int)numericUpDownHistoryLenght.Value)
                {
                    History.Lenght = (int)numericUpDownHistoryLenght.Value;
                }
            }

            #endregion

            STSystem.STController.SetTraymenuIcons(checkBoxUseTraymenuIcons.Checked);

            this.Close();
        }


        #region Hotkeys zuweisen

        /// <summary>
        /// Setzen des Hotkeys fr den gesamten Screen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelSetFullScreen_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (FormHotkeyGrabber _HKGrabber = new FormHotkeyGrabber())
            {
                _HKGrabber.Value = STSystem.CurrentHKFull;

                if (_HKGrabber.ShowDialog(this) == DialogResult.OK)
                {
                    labelHKFullScreenValue.Text = _HKGrabber.CurrentValue;
                    _HotkeyCompScreenBuffer     = _HKGrabber.Value;
                }
            }
        }

        /// <summary>
        /// Setzen des Hotkey fr das aktuelle Fenster
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelSetCurrentScreen_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (FormHotkeyGrabber _HKGrabber = new FormHotkeyGrabber())
            {
                _HKGrabber.Value = STSystem.CurrentHKCurr;

                if (_HKGrabber.ShowDialog(this) == DialogResult.OK)
                {
                    labelHKCurrentScreenValue.Text  = _HKGrabber.CurrentValue;
                    _HotkeyCurrScreenBuffer         = _HKGrabber.Value;
                }
            }
        }

        /// <summary>
        /// Setzen des Hotkeys fr den Auswahlbereich
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelSetDefScreen_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (FormHotkeyGrabber _HKGrabber = new FormHotkeyGrabber())
            {
                _HKGrabber.Value = STSystem.CurrentHKDef;

                if (_HKGrabber.ShowDialog(this) == DialogResult.OK)
                {
                    labelHKDefScreenValue.Text  = _HKGrabber.CurrentValue;
                    _HotkeyDefScreenBuffer      = _HKGrabber.Value;
                }
            }
        }

        #endregion

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormConfig_FormClosing(object sender, FormClosingEventArgs e)
        {
            tabControlConfig.Dispose();
            tabControlConfig = null;

            this.Dispose();
            GC.SuppressFinalize(this);
        }

        #region Hilfebehandlungen

        /// <summary>
        /// Aufrufen der Hilfe
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHelp_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.config);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="hlpevent"></param>
        private void FormConfig_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.config);
        }

        #endregion

        /// <summary>
        /// Setzt MS Paint als Bildbearbeitungsprogramm
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelSetMSPaint_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            textBoxExternalToolName.Text = "Microsoft Paint";
            textBoxExternalToolFile.Text = "mspaint.exe";
        }

        /// <summary>
        /// Setzt den Dateinamen zu dem externen Bildbearbeitungsprogramm
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonSetExternalToolFile_Click(object sender, EventArgs e)
        {
            using (OpenFileDialog _OpenFile = new OpenFileDialog())
            {
                _OpenFile.Title             = STSystem.AppTitle;
                _OpenFile.Filter            = "Programmdateien |*.exe";
                _OpenFile.RestoreDirectory  = true;

                if (_OpenFile.ShowDialog() == DialogResult.OK)
                {
                    textBoxExternalToolFile.Text = _OpenFile.FileName;
                }
            }
        }

        /// <summary>
        /// Wenn der Zustand gendert wird
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBoxEnableHistory_CheckedChanged(object sender, EventArgs e)
        {
            checkBoxHistoryTakeEdited.Enabled = checkBoxEnableHistory.Checked;
            checkBoxHistoryTakeSized.Enabled  = checkBoxEnableHistory.Checked;
            numericUpDownHistoryLenght.Enabled  = checkBoxEnableHistory.Checked;
        }

        /// <summary>
        /// Wenn der Zustand verndert wird
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBoxAutoSaveScreen_CheckedChanged(object sender, EventArgs e)
        {
            comboBoxAutosaveFiletype.Enabled = checkBoxAutoSaveScreen.Checked;
            numericUpDownQualityAutocompress.Enabled = checkBoxAutoSaveScreen.Checked;
        }

        /// <summary>
        /// Setzen des Hotkeys fr den Uploader
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void linkLabelSetUploader_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            using (FormHotkeyGrabber _HKGrabber = new FormHotkeyGrabber())
            {
                _HKGrabber.Value = STSystem.CurrentHKUpload;

                if (_HKGrabber.ShowDialog(this) == DialogResult.OK)
                {
                    labelHKUploadValue.Text = _HKGrabber.CurrentValue;
                    _HotkeyUploadBuffer = _HKGrabber.Value;
                }
            }
        }
    }
}