using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

using Screentaker.Helper;

namespace Screentaker
{
    public partial class FormHistory : Form
    {
        #region Constructor 

        public FormHistory()
        {
            InitializeComponent();
        }

        #endregion

        public Image SelectedImage  = null;
        public bool OpenNew         = false;
        public Image[] _ImagePool   = new Image[0];


        #region Private Methods

        /// <summary>
        /// Hinzufgen der Bilder
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormHistory_Load(object sender, EventArgs e)
        {
            Win32API.User32.AnimateWindow(this.Handle.ToInt32(), 100, 16);

            listViewHistoryImages.Items.Clear();
            imageListLargeHistory.Images.Clear();

            this.Refresh();

            //Jedes Image aus der Arraylist hinzufgen
            int _Counter    = 0;
            _ImagePool      = History.Images;
            foreach (Image _CurrentImage in History.Images)
            {
                imageListLargeHistory.Images.Add(_CurrentImage);
                
                ListViewItem _NewItem = new ListViewItem();
                _NewItem.ImageIndex = _Counter;

                listViewHistoryImages.Items.Add(_NewItem);
               
                _Counter++;
            }

            //Ersten Eintrag auswhlen
            if (listViewHistoryImages.Items.Count > 0)
            {
                listViewHistoryImages.Items[0].Selected = true;
            }

            //Ausblenden des Info-Labels
            LabelPleaseWait.Visible = false;
        }

        /// <summary>
        /// ffnen eines Images aus der History
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listViewHistoryImages_DoubleClick(object sender, EventArgs e)
        {
            //Abbruch falls kein Eintrag ausgewhlt wurde
            if (listViewHistoryImages.SelectedItems.Count == 0) { return; }

            SelectedImage = _ImagePool[listViewHistoryImages.SelectedItems[0].ImageIndex];

            if (OpenNew == true)
            {
                new ImageGrabbing().Show((Image)SelectedImage.Clone(), true);
            }
            else
            {
                this.DialogResult = DialogResult.OK;
            }
        }

        #endregion

        /// <summary>
        /// Zeigt den aktuellen Screen in der Vorschau an
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listViewHistoryImages_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (listViewHistoryImages.SelectedItems.Count == 0)
            {
                return;
            }

            pictureBoxExCurrentSelected.Image = _ImagePool[listViewHistoryImages.SelectedItems[0].ImageIndex];
        }

        /// <summary>
        /// Lschen der History
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonClear_Click(object sender, EventArgs e)
        {
            History.Clear();            
            this.Close();
        }

        /// <summary>
        /// Auswhlen des aktuell ausgewhlten Screenys
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonSelectCurrent_Click(object sender, EventArgs e)
        {
            listViewHistoryImages_DoubleClick(null, null);
        }

        #region Hilfebehandlungen

        /// <summary>
        /// ffnet die spezifische Hilfe
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonHelp_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.history);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="hlpevent"></param>
        private void FormHistory_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.history);
        }

        #endregion
    }
}