using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

using Screentaker.Helper;

namespace Screentaker
{
    public partial class FormUpdate : Form
    {
        #region Internals

        public bool ShowInCorner            = true;
        private string _CurrentVersionFile  = string.Empty;
       
        #endregion

        #region Constructor

        /// <summary>
        /// 
        /// </summary>
        public FormUpdate()
        {
            InitializeComponent();
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormUpdate_Load(object sender, EventArgs e)
        {
            #region Oberflche vorbereiten

            labelReleaseDate.Text       = string.Empty;
            labelCurrentVersion.Text    = string.Empty;
            labelAvailableVersion.Text  = string.Empty;

            #endregion

            this.Refresh();
            Application.DoEvents();

            if (ShowInCorner)
            {
                this.Left = Screen.PrimaryScreen.WorkingArea.Width - this.Width;
                this.Top = Screen.PrimaryScreen.WorkingArea.Height - this.Height;
            }
          
            STSystem.AnimateWindow(this);

            //Beschaffen der Informationen
            if (WebExchange.IsOnline)
            {
                labelCurrentVersion.Text = STSystem.AppVersion + " (" + STSystem.AppVersionAd + ")";

                using (WebExchange _STExchange = new WebExchange(STSystem.InternalSettings.GetValue("WebExchangeURL")))
                {
                    labelAvailableVersion.Text = _STExchange.GetValue("currentversion");

                    string _AVVersionAd = _STExchange.GetValue("currentversionad");
                    if (_AVVersionAd != string.Empty)
                    {
                        labelAvailableVersion.Text += " (" + _AVVersionAd + ")";
                    }

                    labelReleaseDate.Text               = "Release: " + _STExchange.GetValue("currentversionreleased");
                    
                    _CurrentVersionFile                 = _STExchange.GetValue("currentversionfile");

                    linkLabelShowNotes.Command          = _STExchange.GetValue("currentversionnotesurl");
                    linkLabelShowNotes.ExceptionText    = "Die Versionsinformationen konnten nicht bezogen werden.\r\nVersuchen Sie es spter nocheinmal.";
                    
                    //Neue Einstellungen fr die Hilfe etc. setzen
                    STSystem.InternalSettings.SetValue("WebExchangeURL", _STExchange.GetValue("currentupdateurl"));
                    STSystem.InternalSettings.SetValue("OnlineHelpURL", _STExchange.GetValue("CurrentDocumentationURL"));
                    STSystem.InternalSettings.Save();
                }
            }

            //Anpassen der Oberflche
            labelHeadline.Text = "Webupdate - Neue Version gefunden!";
            buttonDownload.Focus();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClose_Click(object sender, EventArgs e)
        {           
            this.Close();
        }

        /// <summary>
        /// Ldt die neue Programmversion herunter
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonDownload_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start(_CurrentVersionFile);
                Environment.Exit(0);
            }
            catch
            {
                Messages.ErrorBox(this, "Die neue Version konnte nicht bezogen werden.\r\nBitte versuchen Sie es spter erneut.");
            }
        }

        #endregion

        #region Hilfebehandlungen

        /// <summary>
        /// Ruft die spezifische Hilfe auf
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHelp_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.webupdate);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="hlpevent"></param>
        private void FormUpdate_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.webupdate);
        }

        #endregion

        /// <summary>
        /// Beim schlieen dieses Fensters wird der Speicher freigegeben
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormUpdate_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Dispose();
            _CurrentVersionFile = null;
        }
    }
}