using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.IO;
using System.Drawing.Imaging;
using System.Threading;
using System.Xml;
using System.Diagnostics;
using System.Text;

using Screentaker.Uploader;
using Screentaker.Helper;

namespace Screentaker
{
    public partial class FormUploadResult : Form
    {
        #region Internals

        private Image _Source           = null;
        private Thread _UploaderThread  = null;
        private string _Filename        = string.Empty;
        private string _ScreenshotLink  = string.Empty;
        
        public UploadType Provider      = UploadType.Imageshack;

        delegate void SetReportCallback(string Value);

        #endregion

        /// <summary>
        /// 
        /// </summary>
        public Image Screenshot
        {
            set
            {
                pictureBoxThumbnail.Image   = value;
                _Source                     = value;
            }
        }

        public FormUploadResult()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Schliet dieses Fenster
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonClose_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Ldt das Image hoch
        /// </summary>
        private void BeginUpload()
        {
            SetReport(new Uploading().UploadEx(_Filename, Provider, ref _ScreenshotLink));
        }

        /// <summary>
        /// Setzt das Ergebnis
        /// </summary>
        /// <param name="Text"></param>
        private void SetReport(string Text)
        {
            try
            {
                // InvokeRequired required compares the thread ID of the
                // calling thread to the thread ID of the creating thread.
                // If these threads are different, it returns true.
                if (this.richTextBoxResult.InvokeRequired)
                {
                    SetReportCallback _Set = new SetReportCallback(SetReport);

                    //Invoke fhrt ein Delegate des zugrundeliegenden Threads aus.
                    //Dabei werden die Parameter als Array bergeben. Hier quasi
                    //als object[]
                    this.Invoke(_Set, new object[] { Text });
                }
                else
                {
                    richTextBoxResult.Text = Text;
                }

                if ((_ScreenshotLink != null) &&(_ScreenshotLink != string.Empty))
                {
                    buttonCopyURL.Enabled = true;
                }
            }
            catch(Exception ex)
            {
                richTextBoxResult.Text = "\r\nWhrend des Uploads ist ein Fehler aufgetreten!\r\n\r\n" + 
                    "Beschreibung: " + ex.Message + "\r\n\r\nMglicherweise gibt es Probleme durch den Provider. Versuchen Sie es spter nocheinmal.";
            }

            buttonClose.Text         = "Schlieen";
        }

        /// <summary>
        /// Wenn der Text gemdert wurde kann der Button zum schliessen angezeigt werden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void richTextBoxResult_TextChanged(object sender, EventArgs e)
        {
            buttonClose.Enabled = true;
        }

        /// <summary>
        /// Wenn das Fenster geladen wird...
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormUploadResult_Load(object sender, EventArgs e)
        {
            STSystem.AnimateWindow(this);

            #region Oberflche anpassen

            if (Provider == UploadType.Imageshack)
            {
                labelDestination.Text = "Ziel: ImageShack";
            }
            else
            {
                labelDestination.Text = "Ziel: DirectUpload";
            }

            this.MinimumSize = this.Size;

            #endregion

            _Filename = STSystem.GetCachedFile(".png");
            
            //Image erzeugen
            string _EncSetting = STSystem.Settings.GetString("UploadFormat");
            ImageProcessing.EncoderByDesc _Encoder = (ImageProcessing.EncoderByDesc)Enum.Parse(typeof(ImageProcessing.EncoderByDesc), _EncSetting);


            ImageProcessing.CompressImage(_Source,
                100,
                _Encoder,
                _Filename);

            _UploaderThread = new Thread(new ThreadStart(BeginUpload));
            _UploaderThread.Start();
        }

        /// <summary>
        /// ffnen einen Link aus dem RTF-Feld
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void richTextBoxResult_LinkClicked(object sender, LinkClickedEventArgs e)
        {
            Process.Start(e.LinkText);
        }

        /// <summary>
        /// Kopiert den Link zu dem Screenshot in die Zwischenablage
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCopyURL_Click(object sender, EventArgs e)
        {
            if (_ScreenshotLink != null)
            {
                Clipboard.SetText(_ScreenshotLink);
            }

            //Fenster schlieen falls gewnscht
            if (STSystem.Settings.GetBool("CloseOnURLLink") == true)
            {
                this.Close();
            }
        }

        #region Hilfebehandlungen

        /// <summary>
        /// Ruft die spezifische Hilfe auf
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHelp_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.uploader);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="hlpevent"></param>
        private void FormUploadResult_HelpRequested(object sender, HelpEventArgs hlpevent)
        {
            STSystem.ProcessHelpRequest(Helper.STSystem.HelpRequestIDs.uploader);
        }

        #endregion

        /// <summary>
        /// Kopiert den ausgewhlten Text in die Zwischenablage
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemCopy_Click(object sender, EventArgs e)
        {
            Clipboard.SetText(richTextBoxResult.SelectedText);
        }

        /// <summary>
        /// Beim schlieen dieses Fensters werden die Ressourcen freigegeben
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void FormUploadResult_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_UploaderThread.ThreadState != System.Threading.ThreadState.Stopped)
            {
                _UploaderThread.Abort();
            }

            //Verwendeten Speicher freigeben
            _Filename = null;
            _ScreenshotLink = null;
            _Source = null;
            _UploaderThread = null;

            this.Dispose();
        }
    }
}