using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing.Printing;
using System.Reflection;
using System.Windows.Forms;
using System.Drawing;
using System.Runtime.InteropServices;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Threading;
using System.Drawing.Imaging;
using Microsoft.Win32;
using System.Xml;

using Screentaker.Uploader;
using Screentaker.Win32API;

namespace Screentaker.Helper
{
    /// <summary>
    /// Liefert alle ntigen Komponenten um Argumente auszuwerten
    /// </summary>
    public class ArgumentHandling
    {
        #region Enums

        /// <summary>
        /// Liste aller Verfgbaren Argumente (Keys)
        /// Diese Werden aber mit - angegeben (-upl)
        /// </summary>
        public enum ArgTypes
        {
            upl, //Upload - ImageShack          
            sfc, //Show Fullscreen
            scc, //Show Currentscreen
            i,    //Tray ausblenden
            g //Grab: cw = CurrentWindow fs = Fullscreen, ss = SelectedScreen, nw = NowWindow
        }

        #endregion

        #region Interne Klassen

        /// <summary>
        /// Klasse die alle Informationen eines Arguments zusammenfasst
        /// </summary>
        public class Argument
        {
            private string _Name = string.Empty;
            private string[] _Values = new string[0];

            /// <summary>
            /// Liefert zurck ob ein bestimmter Parameter gesetzt wurde
            /// </summary>
            /// <param name="SearchValue"></param>
            /// <returns></returns>
            public bool ValueExists(string SearchValue)
            {
                foreach (string _CurrentValue in _Values)
                {
                    if (_CurrentValue.ToLower() == SearchValue.ToLower())
                    {
                        return true;
                    }
                }

                return false;
            }


            /// <summary>
            /// Werte dieses Arguments. Sie werden durch ein Leerzeichen voneinander getrennt
            /// </summary>
            public string[] Values
            {
                set
                {
                    _Values = value;
                }

                get
                {
                    return _Values;
                }
            }

            /// <summary>
            /// Der Name dieses Arguments.. Quasi der Key
            /// </summary>
            public string Name
            {
                get
                {
                    return _Name;
                }

                set
                {
                    if (value.Length < 1)
                    {
                        return;
                    }

                    if (Convert.ToString(value).Substring(0, 1) == "-")
                    {
                        _Name = value.Substring(1);
                    }
                    else
                    {
                        _Name = value;
                    }
                }
            }

            /// <summary>
            /// Liefert das Argument dieses Arguments zurck
            /// </summary>
            public ArgTypes ArgType
            {
                get
                {
                    return (ArgTypes)Enum.Parse(typeof(ArgTypes), _Name);
                }
            }

            /// <summary>
            /// Anzahl der vorhandenen Werte
            /// </summary>
            public int ValueCount
            {
                get
                {
                    return _Values.Length;
                }
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Liefert ein Array aus strings zurck die alle Werte fr ein Argument enthalten
        /// Dabei ist string[0] der Name des Arguments und alle folgenden sind die Parameter
        /// </summary>
        /// <param name="Argument"></param>
        /// <returns></returns>
        public static Argument[] GetArguments(string[] ArgList)
        {
            ArrayList _ReturnArguments  = new ArrayList();
            string _AllValues           = string.Empty;

            #region Fehlerprfungen

            if (ArgList.Length < 2)
            {
                return new Argument[0];
            }

            #endregion

            #region Argumente Zusammenfassen

            for (int _CurrentIndex = 0; _CurrentIndex != ArgList.Length; _CurrentIndex++)
            {
                _AllValues += ArgList[_CurrentIndex] + " ";
            }

            #endregion

            #region Argumente zusammenstellen 

            string[] _Buffer = _AllValues.Split('-');

            foreach (string _CurrentArgument in _Buffer)
            {
                if (_CurrentArgument != string.Empty)
                {
                    string[] _CurrentValues = _CurrentArgument.Split(' ');

                    Argument _NewArg = new Argument();
                    _NewArg.Name = _CurrentValues[0];

                    //Setzen der Werte
                    string[] _ValuesBuffer = new string[(_CurrentValues.Length - 2)];
                    for(int _CurrentVIndex = 0 ; _CurrentVIndex < (_CurrentValues.Length - 2) ; _CurrentVIndex ++)
                    {
                        _ValuesBuffer[_CurrentVIndex] = _CurrentValues[_CurrentVIndex + 1];
                    }
                    _NewArg.Values = _ValuesBuffer;

                    //Neues Argument hinzufgen
                    _ReturnArguments.Add(_NewArg);
                }
            }

            #endregion

            #region Umwandeln der Arrays

            Argument[] _ResultBuffer = new Argument[_ReturnArguments.Count];
            for(int _CurrentIndex = 0 ; _CurrentIndex < _ReturnArguments.Count ; _CurrentIndex ++)
            {
                _ResultBuffer[_CurrentIndex] = (Argument)_ReturnArguments[_CurrentIndex];
            }

            #endregion

            return _ResultBuffer;
        }

        #endregion
    }

    /// <summary>
    /// Bietet Methoden zum Upload von Images
    /// </summary>
    public class Uploading
    {
        #region Public Methods

        /// <summary>
        /// Ldt eine Datei zu einem Provider hoch und liefert den Ergebnistext zurck
        /// </summary>
        /// <param name="Filename"></param>
        /// <param name="?"></param>
        /// <returns></returns>
        public string Upload(string Filename, Uploader.UploadType Provider)
        {
            try
            {
                Uploader.UploadFactory _UploadFactory = UploadFactory.GetInstance;
                Upload _CurrentUpload = _UploadFactory.createUpload(Provider);

                _CurrentUpload.SendPostRequest(Filename);

                //Ergebnis zurckliefern
                string _Result = _CurrentUpload.GetResponse;

                _UploadFactory = null;
                _CurrentUpload = null;

                return _Result;
            }
            catch(Exception ex)
            {
                return "Der Screenshot konnte nicht hochgeladen werden.\r\nMglicherweise ist keine Internetverbindung aufgebaut.\r\n\r\n" + ex.Message;
            }
        }

        /// <summary>
        /// Ldt ein Image hoch und gibt das Ergebnis formatiert wieder zurck
        /// </summary>
        /// <param name="Filename"></param>
        /// <param name="Provider"></param>
        /// <returns></returns>
        public string FormatResult(string UploadResult, Uploader.UploadType Provider, ref string FileLink)
        {
            string _Result = string.Empty;

            try
            {
                //Je nach Upload-Dienst werden entweder eine XML oder
                //eine HTML-Datei ausgewertet
                if (Provider == UploadType.Imageshack)
                {
                    #region Daten einlesen

                    byte[] _DataBuffer = UTF8Encoding.UTF8.GetBytes(UploadResult);

                    MemoryStream _ResultBuffer = new MemoryStream(_DataBuffer.Length);
                    _ResultBuffer.Write(_DataBuffer, 0, _DataBuffer.Length);
                    _ResultBuffer.Position = 0;

                    //Einlesen der XML-Datei
                    XmlDocument _XMLResult = new XmlDocument();
                    _XMLResult.Load(_ResultBuffer);
                    _ResultBuffer.Close();
                    _ResultBuffer.Dispose();

                    _ResultBuffer = null;
                    _DataBuffer = null;

                    XmlElement _RootElement = _XMLResult.DocumentElement;

                    #endregion

                    #region Eintrge verarbeiten

                    _Result = "\r\n";
                    _Result += "Link:\t\t" + _RootElement["ad_link"].InnerText + "\r\n";
                    _Result += "Screenshot:\t" + _RootElement["image_link"].InnerText + "\r\n";
                    _Result += "Thumbnail:\t" + _RootElement["thumb_link"].InnerText + "\r\n";
                    _Result += "\r\n";
                    _Result += "Forum-URL:\t[URL=" + _RootElement["ad_link"].InnerText + "]Screentaker.NET Screenshot[/URL]\r\n";
                    _Result += "Forum-IMG:\t[IMG]" + _RootElement["image_link"].InnerText + "[/IMG]\r\n";
                    _Result += "\r\n";
                    _Result += "Thumbnail:\t" + _RootElement["thumb_exists"].InnerText + "\r\n";
                    _Result += "Servername:\t" + _RootElement["server"].InnerText + "\r\n";
                    _Result += "Dateiname:\t" + _RootElement["image_name"].InnerText + "\r\n";
                    _Result += "Auflsung:\t" + _RootElement["resolution"].InnerText + " Pixel\r\n";
                    _Result += "Dateigre:\t" + _RootElement["filesize"].InnerText + " Byte\r\n";


                    FileLink = _RootElement["image_link"].InnerText;
                    #endregion
                }
                else
                {
                    //Provider = DirectUpload
                    string _cKeyWordA = "Klasse! Ihr Bild wurde erfolgreich auf den Server hochgeladen!";
                    string _cKeyWordB = "Kostenlos Bilder hochladen bei directupload.net";

                    #region Prfen ob die Daten gltig sind

                    if (UploadResult.IndexOf(_cKeyWordA) == -1)
                    {
                        _Result = "\r\nEs ist ein Fehler aufgetreten. Die Daten konnten nicht ausgewertet werden!";
                        return _Result;
                    }

                    #endregion

                    string _ResultBuffer = UploadResult.Substring(UploadResult.IndexOf(_cKeyWordA) + _cKeyWordA.Length,
                        UploadResult.Length - UploadResult.IndexOf(_cKeyWordA) - _cKeyWordA.Length);

                    _ResultBuffer = _ResultBuffer.Substring(0, _ResultBuffer.IndexOf(_cKeyWordB));

                    //Auswerten der Links
                    int _CurrentStart = 0;
                    int _CurrentLenght = 0;

                    #region Eintrge verarbeiten

                    _CurrentStart = _ResultBuffer.IndexOf("<a href=\"") + 9;
                    _CurrentLenght = _ResultBuffer.Substring(_CurrentStart).IndexOf("\" target=\"_blank\">");

                    string _AdLink = _ResultBuffer.Substring(_CurrentStart, _CurrentLenght);

                    _ResultBuffer = _ResultBuffer.Substring(_CurrentStart + _CurrentLenght);
                    _CurrentStart = _ResultBuffer.IndexOf("<img src=\"") + 10;
                    _CurrentLenght = _ResultBuffer.Substring(_CurrentStart).IndexOf("\" border=\"1\">");

                    string _ThumbLink = _ResultBuffer.Substring(_CurrentStart, _CurrentLenght);

                    _ResultBuffer = _ResultBuffer.Substring(_CurrentStart + _CurrentLenght);
                    _CurrentStart = _ResultBuffer.IndexOf("<img src=\"") + 10;
                    _CurrentLenght = _ResultBuffer.Substring(_CurrentStart).IndexOf("\" border=\"1\">");

                    string _ScreenLink = "http://www.directupload.net/images/" + Generators.ShortDate + "/" +
                        _ThumbLink.Substring(_ThumbLink.IndexOf("/temp/") + 6);

                    //Ausgabe des Ergebnisses
                    _Result = "\r\n";
                    _Result += "Link:\t\t" + _AdLink + "\r\n";
                    _Result += "Screenshot:\t" + _ScreenLink + "\r\n";
                    _Result += "Thumbnail:\t" + _ThumbLink + "\r\n";
                    _Result += "\r\n";
                    _Result += "Forum-URL:\t[URL=" + _AdLink + "]Screentaker.NET Screenshot[/URL]\r\n";
                    _Result += "Forum-IMG:\t[IMG]" + _ScreenLink + "[/IMG]";

                    FileLink = _ScreenLink;

                    #endregion
                }
            }
            catch (Exception ex)
            {
                _Result = "\r\nWhrend des Uploads ist ein Fehler aufgetreten!\r\n\r\n" +
                "Beschreibung: " + ex.Message + "\r\n\r\nMglicherweise gibt es Probleme durch den Provider. Versuchen Sie es spter nocheinmal.";
            }

            return _Result;
        }

        /// <summary>
        /// Fhrt einen Completten Updateprozess durch
        /// </summary>
        /// <param name="Filename"></param>
        /// <param name="Provider"></param>
        /// <param name="FileLink"></param>
        /// <returns></returns>
        public string UploadEx(string Filename, Uploader.UploadType Provider, ref string FileLink)
        {
            string _Buffer = Upload(Filename, Provider);
            return FormatResult(_Buffer, Provider, ref FileLink);
        }
        
        #endregion
    }

    /// <summary>
    /// Bietet Methoden die fr die Integration in das Basissystem ntzlich sind
    /// </summary>
    public class OSIntegration
    {
        #region Autorun setzen

        /// <summary>
        /// Setz den Autostart fr ein Programm. Command kann dabei der Dateiname
        /// etc. sein.
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="Command"></param>
        public void CreateAutorun(string Name, string Command)
        {
            using (RegistryKey _RunEntrys = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Run"))
            {
                _RunEntrys.SetValue(Name, Command);
            }
        }

        /// <summary>
        /// Lscht einen Eintrag im Autostart-Bereich der Registry
        /// </summary>
        /// <param name="Name"></param>
        public void DeleteAutorun(string Name)
        {
            using (RegistryKey _RunEntrys = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Run"))
            {
                if (AutorunExists(Name) == true)
                {
                    _RunEntrys.DeleteValue(Name);
                }
            }
        }

        /// <summary>
        /// Liefert zurck ob ein Wert fr Autorun gesetzt wurde
        /// </summary>
        /// <param name="Name"></param>
        /// <returns></returns>
        public bool AutorunExists(string Name)
        {
            using (RegistryKey _RunEntrys = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Run"))
            {
                foreach (string _CurrentName in _RunEntrys.GetValueNames())
                {
                    if (_CurrentName == Name)
                    {
                        return true;
                    }
                }

                return false;
            }
        }

        #endregion
    }

    /// <summary>
    /// Stellt alle Objekte und Methoden bezglich der History bereit
    /// </summary>
    public static class History
    {
        #region Internals

        private static string[] _HistoryFiles = new string[0];
        private static int _HistoryIndex = 0;
        private static int _HistoryLenght = 0;

        #endregion

        #region Properties

        /// <summary>
        /// Liest oder setzt die Lnge der History
        /// </summary>
        public static int Lenght
        {
            get
            {
                return _HistoryLenght;
            }

            set
            {
                _HistoryLenght = value;

                Clear();
            }
        }

        /// <summary>
        /// Liefert ein ImageArray mit allen vorhandenen Images aus der History
        /// </summary>
        /// <returns></returns>
        public static Image[] Images
        {
            get
            {
                ArrayList _ImageListBuffer = new ArrayList();

                //Jedes Image aus der Arraylist hinzufgen
                foreach (string _CurrentFilename in _HistoryFiles)
                {
                    //Prfungen
                    if (File.Exists(_CurrentFilename) == true)
                    {
                        _ImageListBuffer.Add(Image.FromFile(_CurrentFilename));
                    }
                }

                Image[] _Result = new Image[_ImageListBuffer.Count];

                //Konvertieren des Arrays in ein ImmageArray
                for (int _CurrentIndex = 0; _CurrentIndex < _Result.Length; _CurrentIndex++)
                {
                    _Result[_CurrentIndex] = (Image)_ImageListBuffer[_CurrentIndex];
                }

                _ImageListBuffer = null;

                return _Result;
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Fgt der Historie ein neues Bild hinzu
        /// </summary>
        /// <param name="NewImage"></param>
        public static void Add(Image NewImage)
        {
            //Falls die Funktion deaktiviert wurde werden alte Screenshots
            //gelscht und die Objekte gekillt
            if (STSystem.Settings.GetBool("HistoryEnabled"))
            {
                string _Filename = STSystem.GetCachedFile(".bmp");
                NewImage.Save(_Filename, System.Drawing.Imaging.ImageFormat.MemoryBmp);

                //Index zurcksetzen
                if (_HistoryIndex >= _HistoryLenght)
                {
                    _HistoryIndex = 0;
                }

                _HistoryFiles[_HistoryIndex] = _Filename;
                _HistoryIndex++;
            }
        }

        /// <summary>
        /// Setzt die History zurck
        /// </summary>
        public static void Clear()
        {
            foreach (string _CurrentHistoryFile in _HistoryFiles)
            {
                try
                {
                    File.Delete(_CurrentHistoryFile);
                }
                catch { }
            }

            _HistoryFiles = new string[_HistoryLenght];
            _HistoryIndex = 0;
        }

        #endregion
    }

    /// <summary>
    /// Liefert Methoden zum Drucken etc.
    /// </summary>
    public class Printing
    {
        #region Private Methods

        /// <summary>
        /// 
        /// </summary>
        /// <param name="img"></param>
        /// <param name="preview"></param>
        /// <param name="Printername"></param>
        private void Print(Image img, PrintPreviewDialog preview, string Printername)
        {
            using (PrintDocument doc = new PrintDocument())
            {
                // Hier wird das PrintPage-Event in Form einer anonymen Methode erzeugt
                doc.PrintPage += delegate(object sender, PrintPageEventArgs e)
                {
                    Graphics g = e.Graphics;
                    g.PageUnit = GraphicsUnit.Millimeter;
                    g.PageScale = 1F;
                    g.DrawImage(img, new Point());
                };

                // Es wurde eine Vorschau erwnscht
                if (preview != null)
                {
                    doc.PrinterSettings.PrinterName = Printername;

                    preview.Document = doc;
                    preview.ShowDialog();
                }
                // direkt drucken
                else
                {
                    doc.PrinterSettings.PrinterName = Printername;

                    doc.Print();
                }
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Zeigt eine Vorschau des Bildes an.
        /// Anschlieend hat man die Mglichkeit es zu drucken.
        /// </summary>
        /// <param name="img">Das zu druckende Bild</param>
        public void PrintPreview(Image img, string Printername)
        {
            using (PrintPreviewDialog dialog = new PrintPreviewDialog())
            {
                Print(img, dialog, Printername);
            }
        }

        /// <summary>
        /// Druckt sofort das bergebene Bild
        /// </summary>
        /// <param name="img">Das zu druckende Bild</param>
        public void Print(Image img, string Printername)
        {
            Print(img, null);
        }

        #endregion
    }

    /// <summary>
    /// Liefert Methoden etc. um Images zu bearbeiten
    /// </summary>
    public static class ImageProcessing
    {
        #region Enums

        /// <summary>
        /// Enthlt alle Encoderdescriptions
        /// </summary>
        public enum EncoderByDesc
        {
            JPG,
            BMP,
            GIF,
            TIFF,
            PNG
        }
            

        #endregion

        #region Public Methods

        /// <summary>
        /// Liefert an Hand der Extension den entsprechenden Codec
        /// </summary>
        /// <param name="Extension"></param>
        /// <returns></returns>
        public static EncoderByDesc GetEncDescription(string Extension)
        {
            switch (Extension.ToUpper())
            {
                case ".JPG": return EncoderByDesc.JPG;
                case ".PNG": return EncoderByDesc.PNG;
                case ".BMP": return EncoderByDesc.BMP;
                case ".GIF": return EncoderByDesc.GIF;
                case ".TIFF": return EncoderByDesc.TIFF;

                default: return EncoderByDesc.PNG;
            }
        }

        /// <summary>
        /// Liefert ein ImageCodecInfo an Hand des MimeTypes
        /// </summary>
        public static ImageCodecInfo GetEncoderInfo(EncoderByDesc Description)
        {
            string _DescBuffer = Description.ToString();
            if (_DescBuffer == "JPG") { _DescBuffer = "JPEG"; }

            ImageCodecInfo[] _AllCodecs = ImageCodecInfo.GetImageEncoders();
            for (int _CurrentIndex = 0; _CurrentIndex < _AllCodecs.Length; _CurrentIndex++)
            {
                if (_AllCodecs[_CurrentIndex].FormatDescription == _DescBuffer)
                {
                    return _AllCodecs[_CurrentIndex];
                }
            }

            //Keiner gefunden!
            return null;
        } 

        /// <summary>
        /// ffnet den Aktuellen Screenshot in konfigurierten externen Programm.
        /// </summary>
        /// <param name="Screenshot"></param>
        /// <param name="AppFile"></param>
        /// <param name="AppName"></param>
        /// <param name="WaitForExit"></param>
        /// <returns></returns>
        public static Image EditExternal(Image Screenshot, string AppFile, string AppName, bool WaitForExit)
        {
            //Der Screenshot wird zunchst im Temp-Verzeichnis mit einem eindeutigen
            //Dateinamen gespeichert. Danach wird das entsprechende Programm geffnet.
            //Solange das Tool geffnet ist bleibt Screentaker.NET eingefroren.
            //Sobald das Tool beendet wird ldt der Screentaker das Bild aus dem Cache
            //Verzeichnis neu ein und zeigt es an. Danach kann es wie gewohnt gespeichert
            //werden etc.
            string _Filename = STSystem.GetCachedFile(".bmp");
            Screenshot.Save(_Filename, ImageFormat.MemoryBmp);

            #region Starten des Externen Tools
            
            Process _ExternalToolExe = new Process();
            _ExternalToolExe.StartInfo.FileName = AppFile;
            _ExternalToolExe.StartInfo.Arguments = "\"" + _Filename + "\"";

            try
            {
                _ExternalToolExe.Start();
            }
            catch
            {
                MessageBox.Show(AppName + " konnte nicht gestartet werden.",
                    STSystem.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                
                return Screenshot;
            }

            #endregion

            #region Auf Programmende warten

            if (WaitForExit == true)
            {
                _ExternalToolExe.WaitForExit();
            }

            #endregion

            #region Bearbeitetes Bild einlesen und zurckliefern
            
            Image _Result = null;
            try 
            {
                _Result = Image.FromFile(_Filename);
            }
            catch { }

            #endregion

            return _Result;
        }

        /// <summary>
        /// Liefert ein komprimiertes Image zurck.
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="QualityLevel"></param>
        /// <returns></returns>
        public static Image CompressImage(Image Source, int QualityLevel, EncoderByDesc EncoderDesc,  ref double ResultSize, string Filename)
        {
            //Entsprechenden Parameter fr den Encoder bestimmen
            EncoderParameter _QualityParam = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, Convert.ToInt32(QualityLevel));

            //Informationen fr den Codec zusammensetzen
            ImageCodecInfo _IMGCodec = ImageProcessing.GetEncoderInfo(EncoderDesc);
            EncoderParameters _EncParams = new EncoderParameters(1);
            _EncParams.Param[0] = _QualityParam;

            //Image mit der neuen Qualittseinstellung speichern und
            //neu einlesen
            MemoryStream _FileStream = new MemoryStream();
            Source.Save(_FileStream, _IMGCodec, _EncParams);
            ResultSize = _FileStream.Length;
            Bitmap _ImageBuffer = new Bitmap(Image.FromStream(_FileStream));
            _FileStream.Close();

            if (Filename != string.Empty)
            {
                //Falls das Image als JPG gespeichert werden soll.
                if (EncoderDesc == EncoderByDesc.BMP)
                {
                    _ImageBuffer.Save(Filename, ImageFormat.MemoryBmp);
                }
                else
                {
                    _ImageBuffer.Save(Filename, _IMGCodec, _EncParams);
                }
            }

            return _ImageBuffer;
        }

        /// <summary>
        /// Liefert ein komprimiertes Image zurck.
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="QualityLevel"></param>
        /// <returns></returns>
        public static Image CompressImage(Image Source, int QualityLevel)
        {
            double _Buffer = 0;
            return CompressImage(Source, QualityLevel, EncoderByDesc.JPG, ref _Buffer, string.Empty);
        }

        /// <summary>
        /// Liefert ein komprimiertes Image zurck.
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="QualityLevel"></param>
        /// <returns></returns>
        public static Image CompressImage(Image Source, int QualityLevel, EncoderByDesc EncDesc, string Filename)
        {
            double _Buffer = 0;
            return CompressImage(Source, QualityLevel, EncDesc, ref _Buffer, Filename);
        }

        /// <summary>
        /// Liefert ein komprimiertes Image zurck.
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="EncDesc"></param>
        /// <param name="Filename"></param>
        /// <returns></returns>
        public static Image CompressImage(Image Source, EncoderByDesc EncDesc, string Filename)
        {
            return CompressImage(Source, 100, EncDesc, Filename);
        }

        /// <summary>
        /// ffnet einen Dialog zum ffnen einer Bilddatei
        /// </summary>
        /// <param name="Parent"></param>
        /// <param name="InitDir"></param>
        /// <returns></returns>
        public static Image OpenImage(Form Parent, string InitDir)
        {
            using (OpenFileDialog _OpenImage = new OpenFileDialog())
            {
                _OpenImage.Title            = STSystem.AppTitle;
                _OpenImage.Filter           = "Bilddateien |*.jpg;*.jpeg;*.bmp;*.gif;*.png";
                _OpenImage.InitialDirectory = InitDir;
                _OpenImage.CheckFileExists  = true;
                _OpenImage.RestoreDirectory = true;
                
                
                if (_OpenImage.ShowDialog(Parent) == DialogResult.OK)
                {
                    //Prfen auf gltigen Dateityp
                    string _CurrentExtension = new FileInfo(_OpenImage.FileName).Extension;
                    if (! Generators.CheckString(_CurrentExtension, new string[5] { ".jpg", ".jpeg", ".bmp", ".gif", ".png" }))
                    {
                        Messages.ErrorBox(Parent, "Sie haben keinen gltigen Dateitypen angegeben. Ein Upload ist daher nicht mglich.");
                        return null;
                    }

                    //Image zurckgeben
                    try
                    {
                        return Image.FromFile(_OpenImage.FileName);
                    }
                    catch { return null; }

                } else { return null; }
            }
        }

        #endregion
    }

    /// <summary>
    /// Liefert Systemspezifische Methoden
    /// </summary>
    public static class STSystem
    {
        public const string AppTitle = "Screentaker.NET";
        public const string AppVersion = "1.1.0.42";
        public const string AppVersionAd = "";

        private static UserSettings _Settings = null;
        private static UserSettings _SystemSettings = null;
        private static STForm _Controller = null;

        #region Hotkeys

        public static HotkeyHandling.HOTKEY CurrentHKFull;
        public static HotkeyHandling.HOTKEY CurrentHKCurr;
        public static HotkeyHandling.HOTKEY CurrentHKDef;
        public static HotkeyHandling.HOTKEY CurrentHKUpload;

        #endregion

        #region Enums

        /// <summary>
        /// Stellt alle Mglichkeiten fr die Hilfe bereit
        /// </summary>
        public enum HelpRequestIDs
        {
            webupdate,
            editing,
            history,
            qualitywizard,
            uploader,
            main,
            config,
            donate
        }

        #endregion

        #region Properties

        /// <summary>
        /// Controller des ST
        /// </summary>
        public static STForm STController
        {
            get
            {
                return _Controller;
            }

            set
            {
                _Controller = value;
            }
        }

        /// <summary>
        /// Einstellungen des Screentakers fr den aktuellen Benutzer
        /// </summary>
        public static UserSettings Settings
        {
            get
            {
                return _Settings;
            }

            set
            {
                _Settings = value;
            }
        }

        /// <summary>
        /// Bietet den Zugriff auf Benutzerunabhngige Systemeinstellungen
        /// </summary>
        public static UserSettings InternalSettings
        {
            get
            {
                return _SystemSettings;
            }

            set
            {
                _SystemSettings = value;
            }
        }

        /// <summary>
        /// Prft, ob eine neue Version vorliegt.
        /// </summary>
        public static bool NewVersionAvailable
        {
            get
            {
                //Daten ermitteln
                using (WebExchange _WebData = new WebExchange(InternalSettings.GetString("WebExchangeURL")))
                {
                    string _NewVersion      = _WebData.GetValue("currentversion");
                    string _CurrentVersion = STSystem.AppVersion;

                    return CheckVersion(_CurrentVersion, _NewVersion);
                }
            }
        }

        /// <summary>
        /// Liefert den Pfad des Cache-Verzeichnisses zurck
        /// </summary>
        public static string CacheDirectory
        {
            get
            {
                string _ResultDirectory = WorkingDirectory +  @"Cache\";   

                if ( !Directory.Exists(_ResultDirectory))
                { 
                    Directory.CreateDirectory(_ResultDirectory);
                }

                return _ResultDirectory;
            }
        }

        /// <summary>
        /// Liefert das aktuelle Arbeitsverzeichnis zurck
        /// </summary>
        public static string WorkingDirectory
        {
            get
            {
                FileInfo _Result = new FileInfo(Application.ExecutablePath);
                return _Result.Directory.FullName + @"\";
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Fhrt alle Vorgnge aus die beim beenden ausgefhrt werden mssen
        /// </summary>
        public static void Shutdown()
        {
            _Controller.TrayIcon.Visible = false;

            STSystem.DeleteCache();
            HotkeyHandling.UnregisterAll(_Controller);

            while (Application.OpenForms.Count > 0)
            {
                Application.OpenForms[0].Close();
            }

            Environment.Exit(0);
        }

        /// <summary>
        /// Enorm wichtige Methode die alle Komponenten fr den ST initialisiert
        /// </summary>
        public static void Initialize()
        {
            Application.EnableVisualStyles();
            ExceptionHandler.Subscribe();

            #region Systemeinstellungen vorbereiten

            _SystemSettings = new UserSettings(WorkingDirectory,
                "system",
                AppTitle,
                AppVersion);

            #region Standardwerte setzen

            _SystemSettings.AddDefaultValue("ConfigVersion", AppVersion);
            _SystemSettings.AddDefaultValue("UseNoUserprofiles", false);
            _SystemSettings.AddDefaultValue("WebExchangeURL", "http://czaplewski.name/screentaker/webexchange.xml");
            _SystemSettings.AddDefaultValue("OnlineHelpURL", "http://czaplewski.name/screentaker/dokumentation/");

            #endregion

            _SystemSettings.Load();

            #endregion

            #region Benutzereinstellungen vorbereiten

            string _Username = "Screentaker.NET";
            string _UserprofilePath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData);
            if (_SystemSettings.GetBool("UseNoUserprofiles"))
            {
                _Username = "all"; 
                _UserprofilePath = WorkingDirectory;
            }

            _Settings = new UserSettings(_UserprofilePath,
                    _Username,
                    AppTitle,
                    AppVersion);

            #region Standardwerte setzen

            _Settings.AddDefaultValue("ConfigVersion", AppVersion);
            _Settings.AddDefaultValue("ExternalToolName", "Microsoft Paint");
            _Settings.AddDefaultValue("ExternalToolFile", "mspaint.exe");
            _Settings.AddDefaultValue("HistoryLenght", 10);
            _Settings.AddDefaultValue("HistoryEnabled", true);
            _Settings.AddDefaultValue("HomeDir", "Screenshots");
            _Settings.AddDefaultValue("CloseOnAction", true);
            _Settings.AddDefaultValue("AutoUpdate", true);
            _Settings.AddDefaultValue("HotkeyCompScreen", "PrintScreen;False;False;False");
            _Settings.AddDefaultValue("HotkeyCurrScreen", "PrintScreen;True;False;False");
            _Settings.AddDefaultValue("HotkeyDefScreen", "PrintScreen;False;False;True");
            _Settings.AddDefaultValue("AlwaysOpenMaximized", true);
            _Settings.AddDefaultValue("CloseOnURLLink", false);
            _Settings.AddDefaultValue("HistoryTakeEdited", true);
            _Settings.AddDefaultValue("HistoryTakeSized", true);
            _Settings.AddDefaultValue("AutoShowScreen", true);
            _Settings.AddDefaultValue("AutoNumerate", true);
            _Settings.AddDefaultValue("DefaultFilename", "Screenshot_");
            _Settings.AddDefaultValue("AutoSaveScreen", false);
            _Settings.AddDefaultValue("AutoCompressFiletype", "PNG");
            _Settings.AddDefaultValue("QualityAutocompress", 100);
            _Settings.AddDefaultValue("NoLoadAfterEdit", false);
            _Settings.AddDefaultValue("UploadFormat", "PNG");
            _Settings.AddDefaultValue("UseTraymenuIcons", true);
            _Settings.AddDefaultValue("HotkeyUpload", "U;True;True;False");
            _Settings.AddDefaultValue("HotkeyUploadProvider", "ImageShack.us");

            #endregion

            _Settings.Load();

            #endregion

            History.Lenght  = STSystem.Settings.GetInt("HistoryLenght");
            
            #region Hotkeys auslesen

            CurrentHKFull = HotkeyHandling.Generate(Settings.GetString("hotkeycompscreen"));
            CurrentHKCurr = HotkeyHandling.Generate(Settings.GetString("hotkeycurrscreen"));
            CurrentHKDef = HotkeyHandling.Generate(Settings.GetString("hotkeydefscreen"));
            CurrentHKUpload = HotkeyHandling.Generate(Settings.GetString("HotkeyUpload"));

            #endregion
        }

        /// <summary>
        /// Liefert einen Dateinamen fr eine neue Datei im Cache zurck
        /// </summary>
        /// <param name="Extension"></param>
        /// <returns></returns>
        public static string GetCachedFile(string Extension)
        {
            return CacheDirectory + Generators.LongDate + Extension;
        }

        /// <summary>
        /// Liefert einen Dateinamen fr eine neue Datei im Cache zurck. *.dat - Datei.
        /// </summary>
        /// <returns></returns>
        public static string GetCachedFile()
        {
            return GetCachedFile(".dat");
        }

        /// <summary>
        /// Lscht das Cacheverzeichnis
        /// </summary>
        public static void DeleteCache()
        {
            try
            {
                if (Directory.Exists(CacheDirectory))
                {
                    Directory.Delete(CacheDirectory, true);
                    Directory.CreateDirectory(CacheDirectory);
                }
            }
            catch { }
        }

        /// <summary>
        /// Zeigt falls erforderlich das Update-Fenster an. Sollte keine neue Version
        /// vorhanden sein wird wird eine Messagebox angezeigt. Diese wird jedoch nur
        /// angezeigt wenn entsprechend gesetzt
        /// </summary>
        /// <param name="Parent"></param>
        public static void ShowUpdate(Form Parent, bool ShowMessage)
        {
            if (! NewVersionAvailable)
            {
                if (ShowMessage)
                {
                    Messages.InfoBox(Parent, "Es wurde keine neuere Version als die bereits installierte gefunden.");
                }
            }
            else
            {
                using (FormUpdate _Updater = new FormUpdate())
                {
                    _Updater.ShowDialog(Parent);
                }
            }
        }

        /// <summary>
        /// Liefert zurck ob eine Neuere Version wirklich neuer ist als die aktuelle
        /// </summary>
        /// <param name="Current"></param>
        /// <param name="Newer"></param>
        /// <returns></returns>
        public static bool CheckVersion(string Current, string Newer)
        {
            try
            {
                string[] _NewVersionInf = Newer.Split('.');
                string[] _CurrentVersionInf = Current.Split('.');

                //Fehlerprfungen
                if (_NewVersionInf.Length != _CurrentVersionInf.Length)
                {
                    return false;
                }

                //Versionen vergleichen
                bool _VersionIsNewer = false;
                for (int _CurrentIndex = 0; _CurrentIndex < Newer.Length; _CurrentIndex++)
                {
                    int _CurrentValue = Convert.ToInt32(_CurrentVersionInf[_CurrentIndex]);
                    int _NewValue = Convert.ToInt32(_NewVersionInf[_CurrentIndex]);

                    if (_NewValue > _CurrentValue)
                    {
                        _VersionIsNewer = true;
                        break;
                    }
                    else if (_NewValue < _CurrentValue)
                    {
                        break;
                    }
                }

                return _VersionIsNewer;
            }
            catch
            {
                return false;
            }
        }

        /// <summary>
        /// Animiert das angegebene Fenster dem ST Standard entsprechend
        /// Muss im Event FormLoad aufgerufen werden
        /// </summary>
        /// <param name="Target"></param>
        public static void AnimateWindow(Form Target)
        {
            Win32API.User32.AnimateWindow(Target.Handle.ToInt32(), 100, 16);
        }

        /// <summary>
        /// ffnet die Hilfewebseite und ffnet eine bestimmte
        /// HTML-Datei
        /// </summary>
        /// <param name="AreaId"></param>
        public static void ProcessHelpRequest(HelpRequestIDs ID)
        {
            Process.Start(InternalSettings.GetString("OnlineHelpURL") + Convert.ToString(ID) + ".html");
        }

        /// <summary>
        /// Liefert einen Dateinamen zurck, der bisher nicht verwendet wird. Sollte
        /// dabei keine Endung angegeben werden werden alle Endungen bercksichtigt.
        /// Falls eine Datei bereits existiert wird sie aufaddiert. Dabei sind maximal
        /// 999 Dateien mglich!
        /// Falls in den Dateien Lcken zu finden sind wird dennoch die letze Datei verwendet!
        /// </summary>
        /// <param name="dir"></param>
        /// <param name="fileName"></param>
        /// <param name="extension"></param>
        /// <returns></returns>
        public static string GetUnusedFilename(string dir, string fileName, string extension)
        {
            //Liste der vorhandenen Dateinamen erstellen
            ArrayList _ExistingFiles = new ArrayList();

            if (!Directory.Exists(dir))
            {
                Directory.CreateDirectory(dir);
            }

            foreach (FileInfo _CurrentFile in new DirectoryInfo(dir).GetFiles(fileName + "*"))
            {
                _ExistingFiles.Add((extension == string.Empty)
                    ? _CurrentFile.Name.Substring(0, _CurrentFile.Name.Length - 4)
                    : _CurrentFile.Name);
            }

            for (int _CurrentIndex = 0; _CurrentIndex != 999; _CurrentIndex++)
            {
                string _NewFile = fileName + Generators.FillNumber(_CurrentIndex, 999) + extension;

                //Prfen ob die Datei existiert
                bool _Exists = false;
                foreach (string _CurrentValue in _ExistingFiles)
                {
                    if (_CurrentValue == _NewFile)
                    {
                        _Exists = true;
                    }
                }

                if (!_Exists)
                {
                    return _NewFile;
                }
            }

            return string.Empty;
        }

        #endregion
    }

    /// <summary>
    /// Bietet diverse Methoden um schnell und einfach Meldungen auszugeben
    /// </summary>
    public static class Messages
    {
        #region Public Methods

        /// <summary>
        /// Zeigt eine Warnungsbox an
        /// </summary>
        /// <param name="Parent"></param>
        /// <param name="Message"></param>
        public static void WarningBox(Form Parent, string Message)
        {
            MessageBox.Show(Parent, Message, STSystem.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Warning);
        }

        /// <summary>
        /// Zeigt eine Errorbox an
        /// </summary>
        /// <param name="Parent"></param>
        /// <param name="Message"></param>
        public static void ErrorBox(Form Parent, string Message)
        {
            MessageBox.Show(Parent, Message, STSystem.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        /// <summary>
        /// Zeigt eine Infobox an
        /// </summary>
        /// <param name="Parent"></param>
        /// <param name="Message"></param>
        public static void InfoBox(Form Parent, string Message)
        {
            MessageBox.Show(Parent, Message, STSystem.AppTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        #endregion
    }

    /// <summary>
    /// Liefert alle ntigen Methoden etc. um bestimmte Daten zu generieren
    /// oder zu manipulieren
    /// </summary>
    public static class Generators
    {
        #region Properties

        /// <summary>
        /// Liefert einen String des aktuellen Datum im folgenden Format
        /// YYMMTT
        /// </summary>
        public static string ShortDate
        {
            get
            {
                return string.Format("{0:yyMMdd}", DateTime.Now);
            }
        }

        /// <summary>
        /// Liefert einen String des aktuellen Datums + Uhrzeit zurck.
        /// Beispiel: 200704301230220120 (Milisekunden werden ebenfalls bercksichtigt!
        /// </summary>
        public static string LongDate
        {
            get
            {
                return string.Format("{0:yyyyMMddHHmmssfff}", DateTime.Now);
            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Prft ob einer der in Values angegebenen Werte im string Source
        /// enthalten sind
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="Values"></param>
        /// <returns></returns>
        public static bool CheckString(string Source, string[] Values)
        {
            foreach (string _CurrentValue in Values)
            {
                if (Source.ToUpper().IndexOf(_CurrentValue.ToUpper()) > -1)
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Ersetzt alle in dem Array vorkommenden strings mit einem
        /// anderen string
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="Find"></param>
        /// <param name="ReplaceWith"></param>
        /// <returns></returns>
        public static string ReplaceEx(string Source, string[] Find, string ReplaceWith)
        {
            string _Result = Source;

            foreach (string _CurrentFind in Find)
            {
                if (_CurrentFind != null)
                {
                    _Result = _Result.Replace(_CurrentFind, ReplaceWith);
                }
            }

            return _Result;
        }

        /// <summary>
        /// Fllt eine Zahl mit Nullen auf. Dabei wird Max als Begrenzung gesetzt.
        /// Bedeutet: aus 5 bei max 100 wird 005, aus 100 bei Max 1000 wird 0100
        /// </summary>
        /// <param name="Value"></param>
        /// <param name="Max"></param>
        /// <returns></returns>
        public static string FillNumber(int Value, int MaxValue)
        {
            int _MaxDigits      = MaxValue.ToString().Length;
            return  Value.ToString().PadLeft(_MaxDigits, '0');        
        }

        #endregion
    }

    /// <summary>
    /// Stellt alle ntigen Infos und Methoden zur Verfgung die zum grabben
    /// von Images bentigt werden
    /// </summary>
    public class ImageGrabbing
    {
        #region Enums

        /// <summary>
        /// Liefert alle 3 Typen an Screenshots die gegrabbt werden knnen
        /// </summary>
        public enum GrabType
        {
            Selection,
            Full,
            CurrentWindow
        }

        #endregion

        #region Properties

        /// <summary>
        /// Liefert die Position und Gre des aktuellen Fensters zurck. Dies 
        /// kann dann mit GetScreenshot verwendet werden.
        /// </summary>
        /// <returns></returns>
        public Rectangle CurrentWindow
        {
            get
            {
                //Handle des aktuellen Fensters ermitteln
                int _CurrentWindowHandle = Win32API.User32.GetForegroundWindow();

                Win32API.User32.RECT _ClientLocation = new Win32API.User32.RECT();

                //Position und Gre des Fensters ermitteln
                Win32API.User32.GetWindowRect(_CurrentWindowHandle, ref _ClientLocation);

                //Convertieren von Win32API.RECT nach Rectangle
                int _WindowX = _ClientLocation.Left;
                int _WindowY = _ClientLocation.Top;
                int _WindowHeight = _ClientLocation.Bottom - _ClientLocation.Top;
                int _WindowWidth = _ClientLocation.Right - _ClientLocation.Left;

                if (_WindowX < 0) { _WindowX = 0; }
                
                if (_WindowY < 0) 
                {
                    _WindowHeight -= (2* (_WindowY*-1));
                    _WindowY = 0; 
                }

                if (_WindowWidth > Screen.PrimaryScreen.Bounds.Width)
                { _WindowWidth = Screen.PrimaryScreen.Bounds.Width; }

                return new Rectangle(_WindowX, _WindowY, _WindowWidth, _WindowHeight);
            }
        }

        /// <summary>
        /// Liefert ein Rectangle was exact die Screengre entspricht
        /// </summary>
        public Rectangle FullScreen
        {
            get
            {
                return new Rectangle(0,
                                     0,
                                     Screen.PrimaryScreen.Bounds.Width,
                                     Screen.PrimaryScreen.Bounds.Height);

            }
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Diese Methode liefert einen Screenshot. Dabei wird der Bereich
        /// des Screenshots auf dem Desktop in Form eines Rectangles angegeben
        /// </summary>
        /// <param name="Area">Bereich auf dem Desktop</param>
        /// <returns></returns>
        public Image GetScreen(Rectangle Area)
        {
            #region Fehlerprfungen 

            //Damit keine ArgumentException bei einer Breite bzw Hhe des Auswahlrechtsecks
            //geworfen wird, setzen wir diese notfalls auf 1.
            if (Area.Width < 1)
            {
                Area.Width = 1;
            }

            if (Area.Height < 1)
            {
                Area.Height = 1;
            }

            #endregion

            #region Screenshot holen

            Size _ScreenSize = Screen.PrimaryScreen.Bounds.Size;

            IntPtr _hDesktopWindow  = Win32API.User32.GetDesktopWindow();
            IntPtr _hSource         = Win32API.User32.GetWindowDC(_hDesktopWindow);
            IntPtr _hDestination    = Win32API.GDI32.CreateCompatibleDC(_hSource);
            IntPtr _hNewBitmap      = Win32API.GDI32.CreateCompatibleBitmap(_hSource, Area.Width, Area.Height);
            IntPtr _hOldBitmap      = Win32API.GDI32.SelectObject(_hDestination, _hNewBitmap);

            Win32API.GDI32.BitBlt(_hDestination, 0, 0, _ScreenSize.Width, _ScreenSize.Height, _hSource, Area.X, Area.Y, CopyPixelOperation.SourceCopy | CopyPixelOperation.CaptureBlt);

            Bitmap _AlphaScreenshot = Bitmap.FromHbitmap(_hNewBitmap);

            Win32API.GDI32.SelectObject(_hDestination, _hOldBitmap);
            Win32API.GDI32.DeleteObject(_hNewBitmap);
            Win32API.GDI32.DeleteDC(_hDestination);
            Win32API.GDI32.ReleaseDC(_hDestination, _hSource);

            #endregion

            #region Hinzufgen zu der History

            if (STSystem.Settings.GetBool("HistoryEnabled"))
            {
                History.Add(_AlphaScreenshot);
            }

            #endregion

            #region Autospeichern des Screenshots

            if (STSystem.Settings.GetBool("AutoSaveScreen"))
            {
                //Einstellungen einlesen
                string _TargetFile = STSystem.Settings.GetString("HomeDir") + @"\";
                string _DefFilename = STSystem.Settings.GetString("DefaultFilename");
                int _CompressionLevel = STSystem.Settings.GetInt("QualityAutocompress");
                string _Filetype = STSystem.Settings.GetString("AutoCompressFiletype");

                //Vorbereitungen
                if (!Directory.Exists(_TargetFile)) { Directory.CreateDirectory(_TargetFile); }

                _TargetFile += STSystem.GetUnusedFilename(_TargetFile, _DefFilename, "." + _Filetype.ToLower());
                ImageProcessing.CompressImage(_AlphaScreenshot, _CompressionLevel, ImageProcessing.GetEncDescription("." + _Filetype.ToLower()), _TargetFile);                                
            }

            #endregion

            return _AlphaScreenshot;
        }

        /// <summary>
        /// Zeigt einen neuen Screenshot an und bertrgt ihn in die History etc.
        /// </summary>
        /// <param name="Screenshot"></param>
        /// <param name="IgnoreAutoShow"></param>
        /// <returns></returns>
        public void Show(Image Screenshot, bool IgnoreAutoShow)
        {
            //Neuen Screenshot nur falls gewnscht anzeigen
            if ((STSystem.Settings.GetBool("AutoShowScreen")) || (IgnoreAutoShow))
            {
                FormCaptured _NewScreenshot = new FormCaptured();
                _NewScreenshot.Show(Screenshot);
                _NewScreenshot.Activate();
            }
        }

        /// <summary>
        /// ffnet den Auswahlbereich um einen beliebigen Bereich zu grabben
        /// </summary>
        /// <param name="Parent"></param>
        /// <param name="GrabPen"></param>
        /// <param name="Transparency"></param>
        /// <param name="OnlyParent"></param>
        /// <returns></returns>
        public Rectangle SelectArea(Form Parent, Pen GrabPen, double Transparency, bool OnlyParent)
        {
            using (FormScreenSelector _Selector = new FormScreenSelector())
            {
                _Selector.Transparency  = Transparency;
                _Selector.GrabbingPen   = GrabPen;

                if (OnlyParent)
                {
                    if (_Selector.GrabForm(Parent) == DialogResult.OK)
                    {
                        return _Selector.SelectedArea;
                    }
                    else
                    {
                        return Rectangle.Empty;
                    }
                }
                else
                {
                    if (_Selector.Grab(Parent) == DialogResult.OK)
                    {
                        return _Selector.SelectedArea;
                    }
                    else
                    {
                        return Rectangle.Empty;
                    }
                }

                GrabPen.Dispose();
                GrabPen = null;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="Parent"></param>
        /// <returns></returns>
        public Rectangle SelectArea(Form Parent)
        {
            return SelectArea(Parent, new Pen(Color.Black, 1), .5, false);
        }

        /// <summary>
        /// ffnet den Typ an Screenshot direkt im Fenster
        /// </summary>
        /// <param name="Type"></param>
        public void Grab(GrabType Type)
        {
            Rectangle _ImageArea = Rectangle.Empty;

            #region Je nach Typ den Bereich holen

            if (Type == GrabType.Selection)
            {
                _ImageArea = SelectArea(null);
            }

            if (Type == GrabType.Full)
            {
                _ImageArea = FullScreen;
            }

            if (Type == GrabType.CurrentWindow)
            {
                _ImageArea = CurrentWindow;
            }

            #endregion

            if (_ImageArea == Rectangle.Empty)
            {
                //Abbruch falls kein Bereich vorliegt
                return;
            }

            Image _NewImage = GetScreen(_ImageArea);
            Show(_NewImage, false);
        }

        #endregion
    }

    /// <summary>
    /// Stellt alle Methoden etc. bereit die fr das Handling der Hotkeys ntig sind
    /// </summary>
    public static class HotkeyHandling
    {
        #region Internals

        private static ArrayList _Hotkeys = new ArrayList();

        #endregion

        #region Structs

        /// <summary>
        /// Dieses Struct stellt alle ntigen Informationen zu einem Hotkey bereit
        /// </summary>
        public struct HOTKEY
        {
            public ushort ID;
            public Keys Key;
            public bool RequiereAlt;
            public bool RequiereStrg;
            public bool RequiereShift;
            public ushort RequiereCode;
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Liefert ein HOTKEY-Struct zurck welches durch die Message definiert wurde
        /// </summary>
        /// <param name="Msg"></param>
        /// <returns></returns>
        private static HOTKEY GetHotkey(Message Msg)
        {
            int _SendedID = Msg.WParam.ToInt32();
            foreach (HOTKEY _CurrentHotkey in _Hotkeys)
            {
                if (_CurrentHotkey.ID == _SendedID)
                {
                    return _CurrentHotkey;
                }
            }

            return new HOTKEY();
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Registriert einen Globalen Hotkey. Dabei kann Spezifiziert werden welche Keys zudem gedrckt
        /// werden mssen
        /// </summary>
        /// <param name="hotkey"></param>
        /// <param name="mods"></param>
        public static void Register(Form Target, Keys Hotkey, bool ReqAlt, bool ReqStrg, bool ReqShift, ref string Exception)
        {
            ushort _Additions = 0;

            #region Zusatztasten hinzufgen

            if (ReqAlt) { _Additions += 1; }
            if (ReqStrg) { _Additions += 2; }
            if (ReqShift) { _Additions += 4; }

            //Win-Taste = 8

            #endregion

            try
            {
                ushort _CurrentID = Convert.ToUInt16((int)Hotkey + _Additions);

                // Registriere Hotkey, sonst werfe Fehler
                bool _RegSuccess = Win32API.User32.RegisterHotKey(Target.Handle, _CurrentID, _Additions, (int)Hotkey);

                if (! _RegSuccess)
                {
                    Exception = "Konnte Hotkey nicht registrieren. Mglicherweise wird dieser bereits durch ein anderes Programm verwendet.\nError code: " + Marshal.GetLastWin32Error();
                }
                else
                {
                    //Wenn der Hotkey erfolgreich hinzugefgt werden konnte wird
                    //ein neues Hotkey-Struct gefllt und der Arraylist bergeben.
                    HOTKEY _NewHotkey = new HOTKEY();
                    _NewHotkey.ID = _CurrentID;
                    _NewHotkey.Key = Hotkey;
                    _NewHotkey.RequiereAlt = ReqAlt;
                    _NewHotkey.RequiereStrg = ReqStrg;
                    _NewHotkey.RequiereShift = ReqShift;
                    _NewHotkey.RequiereCode = _Additions;

                    _Hotkeys.Add(_NewHotkey);
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message,
                    "Warnung " + e.Message,
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Warning,
                    MessageBoxDefaultButton.Button1);

                UnregisterAll(Target);
            }
        }

        /// <summary>
        /// Enternt alle Hotkeys die einem bestimmten Fenster zugeordnet
        /// wurden
        /// </summary>
        /// <param name="Target"></param>
        public static void UnregisterAll(Form Target)
        {
            foreach (HOTKEY _CurrentHK in _Hotkeys)
            {
                if (_CurrentHK.ID != 0)
                {
                    Win32API.User32.UnregisterHotKey(Target.Handle, _CurrentHK.ID);
                }
            }

            _Hotkeys.Clear();
        }

        /// <summary>
        /// berprft auf einen Hotkey der gedrckt wurde und gibt diesen
        /// als Keys zurck
        /// </summary>
        /// <param name="CurrentMessage"></param>
        /// <returns></returns>
        public static HOTKEY ProcessEvent(Message CurrentMessage)
        {
            //Nur weitermachen wenn es sich um eine Hotkey-Benachrichtigung
            //handelt. Ansonsten abbruch
            if (CurrentMessage.Msg == 0x312)
            {
                return GetHotkey(CurrentMessage);
            }

            return new HOTKEY();
        }

        /// <summary>
        /// Erstellt einen Hotkey an Hand der Values die durch ; getrennt werden
        /// KEYID;ADDITIONS
        /// </summary>
        /// <param name="Values"></param>
        /// <returns></returns>
        public static HOTKEY Generate(string Values)
        {
            HOTKEY _Result = new HOTKEY();
            string[] _Values = Values.Split(';');

            _Result.Key = (Keys)Enum.Parse(typeof(Keys), _Values[0]);
            _Result.RequiereAlt = Convert.ToBoolean(_Values[1]);
            _Result.RequiereStrg = Convert.ToBoolean(_Values[2]);
            _Result.RequiereShift = Convert.ToBoolean(_Values[3]);

            //Errechnen des Addition-Codes
            ushort _AddCode = 0;
            if (Convert.ToBoolean(_Values[1]) == true) { _AddCode += 1; }
            if (Convert.ToBoolean(_Values[2]) == true) { _AddCode += 2; }
            if (Convert.ToBoolean(_Values[3]) == true) { _AddCode += 4; }
            _Result.RequiereCode = _AddCode;

            return _Result;

        }

        /// <summary>
        /// Liefert einen String zurck mit dem wieder ein neuer HOTKEY generiert werden kann
        /// </summary>
        /// <param name="Key"></param>
        /// <returns></returns>
        public static string Generate(HOTKEY Values)
        {
            string _Result = string.Empty;

            _Result += Values.Key.ToString() + ";";
            _Result += Values.RequiereAlt.ToString() + ";";
            _Result += Values.RequiereStrg.ToString() + ";";
            _Result += Values.RequiereShift.ToString();

            return _Result;
        }

        /// <summary>
        /// Liefert zurck ob ein Hotkey-String sowie ein HOTKEY-Struct identisch sind
        /// </summary>
        /// <param name="Source"></param>
        /// <param name="Values"></param>
        /// <returns></returns>
        public static bool Compare(HOTKEY SourceA, HOTKEY SourceB)
        {
            //if (SourceA.Key != SourceB.Key) { return false; }
            //if (SourceA.RequiereAlt != SourceB.RequiereAlt) { return false; }
            //if (SourceA.RequiereStrg != SourceB.RequiereStrg) { return false; }
            //if (SourceA.RequiereShift != SourceB.RequiereShift) { return false; }
            //return true;

            return ((SourceA.Key != SourceB.Key) || (SourceA.RequiereAlt != SourceB.RequiereAlt)
                || (SourceA.RequiereStrg != SourceB.RequiereStrg) || (SourceA.RequiereShift != SourceB.RequiereShift))
                ? false : true;
        }

        #endregion
    }  
}