using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

using Screentaker.Helper;

namespace Screentaker
{
    /// <summary>
    /// Dies ist das unsichtbare Hauptfenster des Programms
    /// </summary>
    public partial class STForm : Form
    {
        #region Constructor

        /// <summary>
        /// Initialisiert eine neue Instanz der Klasse STForm
        /// </summary>
        public STForm()
        {
            this.InitializeComponent();
        }

        #endregion

        /// <summary>
        /// Laden des Fensters, es werden alle ntigen Hotkeys registriert
        /// </summary>
        /// <param name="e"></param>
        protected override void OnLoad(EventArgs e)
        {
            #region Falls gewnscht auf Updates prfen

            if (STSystem.Settings.GetBool("AutoUpdate"))
            {
                STSystem.ShowUpdate(null, false);
            }

            #endregion
           
            RegisterHotkeys();
            SetTraymenuIcons(STSystem.Settings.GetBool("UseTraymenuIcons"));
            TrayIcon.Visible = true;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="m"></param>
        protected override void WndProc(ref Message _CurrentMessage)
        {
            base.WndProc(ref _CurrentMessage);
            HotkeyHandling.HOTKEY _Value = HotkeyHandling.ProcessEvent(_CurrentMessage);

            if (_Value.Key == Keys.None)
            {
                //Abbruch da es sich um keinen Hotkey handelte
                return;
            }
            else
            {
                //Aktuelles Fenster anzeigen
                if (HotkeyHandling.Compare(_Value, STSystem.CurrentHKCurr))
                {
                    new ImageGrabbing().Grab(ImageGrabbing.GrabType.CurrentWindow);
                }

                //Freien Bereich selektieren
                if (HotkeyHandling.Compare(_Value, STSystem.CurrentHKDef))
                {
                    new ImageGrabbing().Grab(ImageGrabbing.GrabType.Selection);
                }

                //Gesamten Bildschirm capturen
                if (HotkeyHandling.Compare(_Value, STSystem.CurrentHKFull))
                {
                    new ImageGrabbing().Grab(ImageGrabbing.GrabType.Full);
                }

                if (HotkeyHandling.Compare(_Value, STSystem.CurrentHKUpload))
                {
                    string _Provider = STSystem.Settings.GetString("HotkeyUploadProvider").ToLower();

                    switch (_Provider)
                    {
                        case "imageshack.us": UploadTo(Uploader.UploadType.Imageshack); break;
                        case "directupload.net": UploadTo(Uploader.UploadType.DirectUpload); break;
                    }
                }
            }
        }

        #region Public Methods

        /// <summary>
        /// Setzen der Icons im Traymenu
        /// </summary>
        /// <param name="State"></param>
        public void SetTraymenuIcons(bool State)
        {
            if (State == true)
            {
                this.openToolStripMenuItem.Image = global::Screentaker.Properties.Resources.NewDocumentHS;
                this.MenuItemHistory.Image = global::Screentaker.Properties.Resources.HistoryHS;
                this.MenuItemUploader.Image = global::Screentaker.Properties.Resources.UploadHS;
                this.ConfigToolStripMenuItem.Image = global::Screentaker.Properties.Resources.PropertiesHS;
                this.MenuItemDonate.Image = global::Screentaker.Properties.Resources.DonationHS_16x16;
                this.MenuItemHelp.Image = global::Screentaker.Properties.Resources.HelpHS;
                this.MenuItemOpen.Image = global::Screentaker.Properties.Resources.OpenHS;
            }
            else
            {
                foreach (ToolStripItem _CurrentItem in contextMenu.Items)
                {
                    if (_CurrentItem.Image != null)
                    {
                        _CurrentItem.Image.Dispose();
                        _CurrentItem.Image = null;
                    }
                }
            }
        }

        /// <summary>
        /// Registriert alle Hotkeys die dem ST zugeordnet wurden
        /// </summary>
        public void RegisterHotkeys()
        {
            string _ExceptionResult = string.Empty;

            HotkeyHandling.Register(this,
                STSystem.CurrentHKFull.Key,
                STSystem.CurrentHKFull.RequiereAlt,
                STSystem.CurrentHKFull.RequiereStrg,
                STSystem.CurrentHKFull.RequiereShift,
                ref _ExceptionResult);

            HotkeyHandling.Register(this,
                STSystem.CurrentHKCurr.Key,
                STSystem.CurrentHKCurr.RequiereAlt,
                STSystem.CurrentHKCurr.RequiereStrg,
                STSystem.CurrentHKCurr.RequiereShift,
                ref _ExceptionResult);

            HotkeyHandling.Register(this,
               STSystem.CurrentHKDef.Key,
               STSystem.CurrentHKDef.RequiereAlt,
               STSystem.CurrentHKDef.RequiereStrg,
               STSystem.CurrentHKDef.RequiereShift,
               ref _ExceptionResult);

            HotkeyHandling.Register(this,
               STSystem.CurrentHKUpload.Key,
               STSystem.CurrentHKUpload.RequiereAlt,
               STSystem.CurrentHKUpload.RequiereStrg,
               STSystem.CurrentHKUpload.RequiereShift,
               ref _ExceptionResult);


            if (_ExceptionResult != string.Empty)
            {
                TrayIcon.Visible = true;
                TrayIcon.ShowBalloonTip(0, STSystem.AppTitle, _ExceptionResult, ToolTipIcon.Warning);
            }
        }

        #endregion

        #region Private Methods

        /// <summary>
        /// Klick auf Neu ffnet auswahlfenster und zeigt Ergebnis an
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Open_Click(object sender, EventArgs e)
        {
            new ImageGrabbing().Grab(ImageGrabbing.GrabType.Selection);
        }

        /// <summary>
        /// Klick im Tray auf Beenden
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Close_Click(object sender, EventArgs e)
        {
            TrayIcon.Visible = false;
            STSystem.Shutdown();
        }

        /// <summary>
        /// ffnet den Einstellungen-Dialog und ldt die ntigen Einstellungen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            using (FormConfig _Configuration = new FormConfig())
            {
                _Configuration.StartPosition = FormStartPosition.Manual;
                _Configuration.Left = Screen.PrimaryScreen.WorkingArea.Width - _Configuration.Width;
                _Configuration.Top = Screen.PrimaryScreen.WorkingArea.Height - _Configuration.Height;
                _Configuration.ShowInTaskbar = true;

                _Configuration.ShowDialog();
            }
        }

        /// <summary>
        /// Verlauf anzeigen
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemHistory_Click(object sender, EventArgs e)
        {
            FormHistory _History = new FormHistory();
            _History.OpenNew = true;
            _History.Show();
        }

        /// <summary>
        /// Wenn mit der Maus auf das Traysymbol geklickt wurde wird der Auswahlbereich
        /// geffnet
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void taskIcon_MouseClick(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
            {
                new ImageGrabbing().Grab(ImageGrabbing.GrabType.Selection);
            }
        }

        /// <summary>
        /// Zeigt das Info-Form ber dieses Programm an
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemST_Click(object sender, EventArgs e)
        {
            using (FormAbout _About = new FormAbout())
            {
                _About.ShowInCorner = true;
                _About.ShowInTaskbar = true;
                _About.ShowDialog(this);
            }
        }

        /// <summary>
        /// Upload zu DirectUpload.NET
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemUplDirectUpload_Click(object sender, EventArgs e)
        {
            UploadTo(Uploader.UploadType.DirectUpload);
        }

        /// <summary>
        /// Direktes hochladen einer Datei
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemUplImageShack_Click(object sender, EventArgs e)
        {
            UploadTo(Uploader.UploadType.Imageshack);
        }

        /// <summary>
        /// Ldt eine beliebige Datei an einem bestimmten Provider hoch
        /// </summary>
        /// <param name="Provider"></param>
        private void UploadTo(Uploader.UploadType Provider)
        {
            Image _UploadImage = ImageProcessing.OpenImage(null, string.Empty);

            if (_UploadImage != null)
            {
                using (FormUploadResult _Upload = new FormUploadResult())
                {
                    _Upload.Screenshot = _UploadImage;
                    _Upload.Provider = Provider;
                    _Upload.StartPosition = FormStartPosition.Manual;

                    _Upload.Left = Screen.PrimaryScreen.WorkingArea.Width - _Upload.Width;
                    _Upload.Top = Screen.PrimaryScreen.WorkingArea.Height - _Upload.Height;

                    _Upload.TopMost         = false;
                    _Upload.ShowInTaskbar   = true;
                    _Upload.ShowDialog();
                }
            }
        }

        /// <summary>
        /// Ermglicht eine Spende
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemDonate_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(STSystem.HelpRequestIDs.donate);
        }

        /// <summary>
        /// Aufrufen der Online-Hilfe fr Screentaker.NET
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemHelp_Click(object sender, EventArgs e)
        {
            STSystem.ProcessHelpRequest(STSystem.HelpRequestIDs.main);
        }

        #endregion

        /// <summary>
        /// ffnen eines neuen Screenshots
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void MenuItemOpen_Click(object sender, EventArgs e)
        {
            Image _OpenedImage = ImageProcessing.OpenImage(null, STSystem.Settings.GetString("HomeDir"));

            if(_OpenedImage != null)
            {
                //Aktion durchfhren
                new ImageGrabbing().Show(_OpenedImage, true);    
            }
        }
    }
}