using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Diagnostics;
using System.Data;
using System.Collections;
using System.Net;

namespace Screentaker
{
    /// <summary>
    /// Diese Klasse stellt alle Einstellungen bereit die Benutzerspezifisch gespeichert werden.
    /// </summary>
    public class UserSettings : IDisposable
    {
        public class Setting
        {
            public string Name;
            public string Value;
        }

        #region Internals

        private ArrayList _SettingContainer     = new ArrayList();
        private ArrayList _DefaultValues        = new ArrayList();

        private string _Filename                = string.Empty;
        private string _CurrentUser             = string.Empty;
        private string _ConfigVersion           = "0.0.0.0";
        private string _AppTitle                = "none";

        private string _ProfilesRoot              = @".\Profiles";

        #endregion

        #region Contructor

        /// <summary>
        /// Die Usereinstellungen werden nur 1x instanziiert und dann im Speicher verwaltet.
        /// Wenn das Programm beendet wird, knnen sie gespeichert werden
        /// </summary>
        /// <param name="Filename"></param>
        /// <param name="Username"></param>
        /// <param name="ProfilesPath"></param>
        public UserSettings(string ProfilesPath, string Username, string AppTitle, string ConfigVersion)
        {
            #region Verzeichnis setzen

            _ProfilesRoot = ProfilesPath;

            if (!Directory.Exists(_ProfilesRoot)) { Directory.CreateDirectory(_ProfilesRoot); }
            if (! _ProfilesRoot.EndsWith(@"\")) { _ProfilesRoot += @"\"; }

            #endregion

            _Filename           = _ProfilesRoot + Username + ".cfg";
            _CurrentUser        = Username;
            _AppTitle           = AppTitle;
            _ConfigVersion      = ConfigVersion;

            #if(DEBUG)
            _ConfigVersion      = "0.0.0.0";
            #endif
        }

        #endregion

        #region Private Methods

       /// <summary>
        /// Liefert die Information ob eine bestimmte Section in einer bestimmten 
        /// Tabelle existiert.
        /// </summary>
        /// <param name="Keyname"></param>
        /// <returns></returns>
        private bool SettingExists(string Keyname)
        {
            foreach(Setting _CurrentSetting in _SettingContainer)
            {
                if (_CurrentSetting.Name.ToLower() == Keyname.ToLower())
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// Diese Methode erstellt eine neue Spalte in der Usersettings-Tabelle.
        /// Sollte der User bereits existieren wird abgebrochen. Die Werte fr
        /// den neuen Benuter werden bereits mit Zufallsdaten gefllt.
        /// </summary>
        /// <param name="Username"></param>
        /// <returns></returns>
        private void CreateProfile(string Username)
        {
            _Filename = _ProfilesRoot + Username + ".cfg";
            _SettingContainer = new ArrayList();

            foreach (Setting _CurrentSetting in _DefaultValues)
            {
                Add(_CurrentSetting);
            }
                  
            Save();
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Ldt die Einstellungen fr das angegebene Profil
        /// </summary>
        public void Load()
        {
            RefreshContent();
        }

        /// <summary>
        /// Liefert zurck ob eine bestimmte Benutzertabelle bereits existiert
        /// </summary>
        /// <param name="Username"></param>
        /// <returns></returns>
        public bool ProfileExists(string Username)
        {
            if (!Directory.Exists(_ProfilesRoot)) { Directory.CreateDirectory(_ProfilesRoot); }
            return File.Exists(_ProfilesRoot + Username + ".cfg");
        }

        /// <summary>
        /// Fgt den Einstellungen eine neue Einstellung hinzu
        /// </summary>
        /// <param name="NewSetting"></param>
        public void Add(Setting NewSetting)
        {
            if (! SettingExists(NewSetting.Name))
            {
                _SettingContainer.Add(NewSetting);
            }
        }

        /// <summary>
        /// Fgt den Einstellungen eine neue Einstellung hinzu
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="Value"></param>
        public void Add(string Name, object Value)
        {
            Setting _NewSetting = new Setting();
            _NewSetting.Name    = Name;
            _NewSetting.Value   = Convert.ToString(Value);

            Add(_NewSetting);
        }

        /// <summary>
        /// Fgt den Einstellungen eine neue Einstellung hinzu
        /// </summary>
        /// <param name="Name"></param>
        /// <param name="Value"></param>
        public void AddDefaultValue(string Name, object Value)
        {
            Setting _NewSetting = new Setting();
            _NewSetting.Name = Name;
            _NewSetting.Value = Convert.ToString(Value);

            _DefaultValues.Add(_NewSetting);
        }


        /// <summary>
        /// Liefert der Wert eines bestimmten Keys zurck. Dabei werden die Tabelle sowie die
        /// Row in Form einer Section angegeben. Dabei wird der Username als Section verwendet.
        /// Bei den Systemeinstellungen wird DEFAULT als Section verwendet
        /// </summary>
        /// <param name="Keyname"></param>
        /// <param name="ErrorValue"></param>
        /// <returns></returns>
        public string GetValue(string Name, string ErrorValue)
        {
            try
            {
                foreach(Setting _CurrentSetting in _SettingContainer)
                {
                    if (_CurrentSetting.Name.ToLower() == Name.ToLower())
                    {
                        return _CurrentSetting.Value;
                    }
                }

                return "<null>";
            }
            catch
            {
                //Im Fehlerfall wird ein leerer String zurckgeliefert
                return ErrorValue;
            }
        }

        /// <summary>
        /// Liefert einen Wert fr diesen Benutzer
        /// </summary>
        /// <param name="Name"></param>
        /// <returns></returns>
        public string GetValue(string Name)
        {
            return GetValue(Name, string.Empty);
        }

        /// <summary>
        /// Liefert denn bool-Wert einer Benutzereinstellung zurck
        /// </summary>
        /// <param name="Keyname"></param>
        /// <returns></returns>
        public bool GetBool(string Keyname)
        {
            return Convert.ToBoolean(GetValue(Keyname));
        }

        /// <summary>
        /// Liefert den String-Wert einer Benutzereinstellung zurck
        /// </summary>
        /// <param name="Keyname"></param>
        /// <returns></returns>
        public string GetString(string Keyname)
        {
            return Convert.ToString(GetValue(Keyname));
        }

        /// <summary>
        /// Liefert den Int-Wert einer Benutzereinstellung zurck
        /// </summary>
        /// <param name="Keyname"></param>
        /// <returns></returns>
        public int GetInt(string Keyname)
        {
            return Convert.ToInt32(GetValue(Keyname));
        }

        /// <summary>
        /// Speichert eine bestimmte Usereinstellung
        /// </summary>
        /// <param name="Keyname"></param>
        /// <param name="Value"></param>
        public void SetValue(string Keyname, object Value)
        {
            foreach (Setting _CurrentSetting in _SettingContainer)
            {
                if (_CurrentSetting.Name.ToLower() == Keyname.ToLower())
                {
                    _CurrentSetting.Value = Convert.ToString(Value);
                    break;
                }
            }

            //Falls der Wert noch nicht existiert wird er angelegt
            Add(Keyname, Value);
        }

        /// <summary>
        /// Liest das DataSet neu ein
        /// </summary>
        public void RefreshContent()
        {
            if (!Directory.Exists(_ProfilesRoot)) { Directory.CreateDirectory(_ProfilesRoot); }
            if (! ProfileExists(_CurrentUser)) { CreateProfile(_CurrentUser); }

            _SettingContainer = new ArrayList();

            StreamReader _SReader = new StreamReader(_Filename);
            string _CurrentLine = _SReader.ReadLine();

            while (_CurrentLine != null)
            {
                //Eintrag verarbeiten
                if ((_CurrentLine.IndexOf("=") > -1) && (_CurrentLine.Substring(0,2) != "--"))
                {
                    Setting _NewSetting = new Setting();
                    _NewSetting.Name = _CurrentLine.Substring(0, _CurrentLine.IndexOf("="));
                    _NewSetting.Value = _CurrentLine.Substring(_CurrentLine.IndexOf("=") + 1);
                    _SettingContainer.Add(_NewSetting);
                }
           
                _CurrentLine = _SReader.ReadLine();
            }

            _SReader.Close();
            _SReader.Dispose();

            //Mergen der alten Einstellungen, falls ntig
            if (GetString("ConfigVersion") != _ConfigVersion)
            {
                ArrayList _Buffer = _SettingContainer;
                CreateProfile(_CurrentUser);

                foreach (Setting _CurrBufferSetting in _Buffer)
                {
                    SetValue(_CurrBufferSetting.Name, _CurrBufferSetting.Value);
                }
            }
        }

        /// <summary>
        /// Speichert alle Einstellungen in der angegebenen XML-Datei
        /// </summary>
        public void Save()
        {
            if (!Directory.Exists(_ProfilesRoot)) { Directory.CreateDirectory(_ProfilesRoot); }

            StreamWriter _SWriter = new StreamWriter(_Filename);
            _SWriter.WriteLine("[" + _AppTitle + "]");

            foreach (Setting _CurrentSetting in _SettingContainer)
            {
                _SWriter.WriteLine(_CurrentSetting.Name + "=" + _CurrentSetting.Value);
            }

            _SWriter.Close();
            _SWriter.Dispose();
        }

        #endregion

        #region Properties

        /// <summary>
        /// Setzt den aktuellen Benutzer
        /// </summary>
        public string CurrentUser
        {
            get
            {
                return _CurrentUser;
            }
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            _SettingContainer.Clear();
            _SettingContainer = null;

            _Filename         = null;
            _CurrentUser      = null;
            _ConfigVersion    = null;
            _AppTitle         = null;
        }

        #endregion
    }
}
