using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.IO;
using System.Xml;

namespace Screentaker
{
    /// <summary>
    /// Ermglicht es Einstellungen ber das Internet abzurufen. Diese mssen in Form einer
    /// XML-Datei vorliegen.
    /// </summary>
    public class WebExchange : IDisposable
    {
        private string _URL                 = string.Empty;
        private byte[] _Data                = new byte[0];
        private bool _ValidData             = false;
        private Exception _LastEx           = null;

        private WebClient _Client           = null;
        private XmlDocument _XMLDoc         = null;
        private XmlElement _XMLRoot         = null;

        #region Constructor

        /// <summary>
        /// Instanziiert die Klasse und ldt automatisch die Daten der angegebenen XML-
        /// Datei.
        /// </summary>
        /// <param name="XMLURL"></param>
        public WebExchange(string XMLURL)
        {
            _URL = XMLURL;
            RefreshContent();
        }

        #endregion

        #region Public Methods

        /// <summary>
        /// Liest den Inhalt der XML-Datei neu ein
        /// </summary>
        public void RefreshContent()
        {
            try
            {
                _Client = new WebClient();
                _XMLDoc = new XmlDocument();

                _Data = _Client.DownloadData(_URL);
              
                //Daten in einen MemoryStream schreiben. Nachdem die Daten geschrieben
                //wurden muss die Position wieder auf 0 gesetzt werden, da das 
                //XMLDocument an der entsprechenden Position beginnt zu lesen!!
                MemoryStream _BufferStream = new MemoryStream(_Data.Length);
                _BufferStream.Write(_Data, 0 , _Data.Length);
                _BufferStream.Position = 0;
                
                //Einlesen der Daten
                _XMLDoc.Load(_BufferStream);
                
                _BufferStream.Close();
                _BufferStream.Dispose();

                _XMLRoot = _XMLDoc.DocumentElement;
          
                _ValidData = true;
            }
            catch(Exception ex)
            {
                _ValidData  = false;
                _LastEx     = ex;
            }
        }

        /// <summary>
        /// Liefert den Wert eines bestimmten Schlssels zurck
        /// </summary>
        /// <param name="Keyname"></param>
        /// <returns>Im Gehlerfall: NULL</returns>
        public string GetValue(string Keyname)
        {
            try
            {
                return _XMLRoot[Keyname].InnerText;
            }
            catch
            {
                return null;
            }
        }

        #endregion

        #region Properties

        /// <summary>
        /// Liefgert die Standard-URL der XML-Datei zurck
        /// </summary>
        public string URL
        {
            get
            {
                return _URL;
            }

            set
            {
                _URL = value;
            }
        }

        /// <summary>
        /// Liefert alle Werte die in derExchange-Datei vorliegen
        /// </summary>
        public XmlElement Values
        {
            get
            {
                return _XMLRoot;
            }
        }

        /// <summary>
        /// Liefert zurck ob gltige Daten vorliegen
        /// </summary>
        public bool Valid
        {
            get
            {
                return _ValidData;
            }
        }

        /// <summary>
        /// Liefert die zuletzt aufgetretene Exception zurck
        /// </summary>
        public Exception LastException
        {
            get
            {
                return _LastEx;
            }
        }

        #endregion

        #region Properties (Static)

        /// <summary>
        /// Liefert zurck ob der Client zugriff auf das Internet hat.
        /// </summary>
        public static bool IsOnline
        {
            get
            {
                try
                {
                    WebClient _TestClient = new WebClient();
                    _TestClient.DownloadData("http://www.google.de");

                    return true;
                }
                catch
                {
                    return false;
                }
            }
        }

        #endregion

        #region IDisposable Members

        /// <summary>
        /// Freigeben des Speichers
        /// </summary>
        public void Dispose()
        {
            _URL        = null;
            _Client.Dispose();
            _Client     = null;
            _Data       = null;
            _LastEx     = null;
            _XMLRoot    = null;
            _XMLDoc     = null;

            GC.Collect();
        }

        #endregion
    }
}
