using System;
using System.Runtime.InteropServices;
using System.Drawing;

namespace Screentaker.Win32API
{
    /// <summary>
    /// Methoden der GDI32.dll
    /// </summary>
    public class GDI32
    {
        /// <summary>
        /// The BitBlt function performs a bit-block transfer of the color
        /// data corresponding to a rectangle of pixels from the specified source
        /// device context into a destination device context.
        /// </summary>
        /// <param name="hdcDest"></param>
        /// <param name="xDest"></param>
        /// <param name="yDest"></param>
        /// <param name="wDest"></param>
        /// <param name="hDest"></param>
        /// <param name="hdcSource"></param>
        /// <param name="xSrc"></param>
        /// <param name="ySrc"></param>
        /// <param name="rop"></param>
        /// <returns></returns>
        [DllImport("gdi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool BitBlt(IntPtr hdcDest, int xDest, int yDest, int
        wDest, int hDest, IntPtr hdcSource, int xSrc, int ySrc, CopyPixelOperation rop);
        
        /// <summary>
        /// The ReleaseDC function releases a device context (DC), freeing
        /// it for use by other applications. The effect of the ReleaseDC 
        /// function depends on the type of DC. It frees only common and window DCs.
        /// It has no effect on class or private DCs.
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="hDc"></param>
        /// <returns></returns>
        [DllImport("user32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern bool ReleaseDC(IntPtr hWnd, IntPtr hDc);
        
        /// <summary>
        /// The DeleteDC function deletes the specified device context (DC).
        /// </summary>
        /// <param name="hDc"></param>
        /// <returns></returns>
        [DllImport("gdi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr DeleteDC(IntPtr hDc);
        
        /// <summary>
        /// The DeleteObject function deletes a logical pen, brush, font, 
        /// bitmap, region, or palette, freeing all system resources associated
        /// with the object. After the object is deleted, the specified handle
        /// is no longer valid.
        /// </summary>
        /// <param name="hDc"></param>
        /// <returns></returns>
        [DllImport("gdi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr DeleteObject(IntPtr hDc);
        
        /// <summary>
        /// The CreateCompatibleBitmap function creates a bitmap 
        /// compatible with the device that is associated with the 
        /// specified device context.
        /// </summary>
        /// <param name="hdc"></param>
        /// <param name="nWidth"></param>
        /// <param name="nHeight"></param>
        /// <returns></returns>
        [DllImport("gdi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr CreateCompatibleBitmap(IntPtr hdc, int nWidth, int nHeight);
        
        /// <summary>
        /// The CreateCompatibleDC function creates a memory device context (DC)
        /// compatible with the specified device.
        /// </summary>
        /// <param name="hdc"></param>
        /// <returns></returns>
        [DllImport("gdi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr CreateCompatibleDC(IntPtr hdc);
        
        /// <summary>
        /// The SelectObject function selects an object into the specified 
        /// device context (DC). The new object replaces the previous 
        /// object of the same type.
        /// </summary>
        /// <param name="hdc"></param>
        /// <param name="bmp"></param>
        /// <returns></returns>
        [DllImport("gdi32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern IntPtr SelectObject(IntPtr hdc, IntPtr bmp);
    }

    /// <summary>
    /// Native Methoden aus der Win32 API
    /// </summary>
    public class User32
    {
        /// <summary>
        /// left Specifies the x-coordinate of the upper-left corner of the rectangle. 
        /// top Specifies the y-coordinate of the upper-left corner of the rectangle. 
        /// right Specifies the x-coordinate of the lower-right corner of the rectangle. 
        /// bottom Specifies the y-coordinate of the lower-right corner of the rectangle.
        /// </summary>
        public struct RECT
        {
            public int Left;
            public int Top;
            public int Right;
            public int Bottom;
        }

        /// <summary>
        /// The GetDesktopWindow function returns a handle to the
        /// desktop window. The desktop window covers the entire screen.
        /// The desktop window is the area on top of which other windows
        /// are painted.
        /// </summary>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern IntPtr GetDesktopWindow();
        
        /// <summary>
        /// The GetWindowDC function retrieves the device context (DC) for the entire window,
        /// including title bar, menus, and scroll bars. A window device context permits 
        /// painting anywhere in a window, because the origin of the device context 
        /// is the upper-left corner of the window instead of the client area.
        /// </summary>
        /// <param name="ptr"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern IntPtr GetWindowDC(IntPtr ptr);
        
        /// <summary>
        /// This function defines a system-wide hot key.
        /// </summary>
        /// <param name="hwnd"></param>
        /// <param name="id"></param>
        /// <param name="fsModifiers"></param>
        /// <param name="vk"></param>
        /// <returns></returns>
        [DllImport("user32", SetLastError = true)]
        public static extern bool RegisterHotKey(IntPtr hwnd, int id, int fsModifiers, int vk);
        
        /// <summary>
        /// The UnregisterHotKey function frees a hot key previously 
        /// registered by the calling thread.
        /// </summary>
        /// <param name="hwnd"></param>
        /// <param name="id"></param>
        /// <returns></returns>
        [DllImport("user32", SetLastError = true)]
        public static extern bool UnregisterHotKey(IntPtr hwnd, int id);
        
        /// <summary>
        /// The GetWindowRect function retrieves the dimensions of the bounding 
        /// rectangle of the specified window. The dimensions are given in screen 
        /// coordinates that are relative to the upper-left corner of the screen.
        /// </summary>
        /// <param name="hWnd"></param>
        /// <param name="lpRect"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern long GetWindowRect(int hWnd, ref RECT lpRect);

        /// <summary>
        /// Liefert den Handle des Aktiven Fensters zurck.
        /// </summary>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern int GetForegroundWindow();

        /// <summary>
        /// The AnimateWindow function enables you to produce 
        /// special effects when showing or hiding windows. 
        /// There are four types of animation: roll, slide, 
        /// collapse or expand, and alpha-blended fade.
        /// </summary>
        /// <param name="hwnd"></param>
        /// <param name="dwTime"></param>
        /// <param name="dwFlags"></param>
        /// <returns></returns>
        [DllImport("user32.dll")]
        public static extern bool AnimateWindow(int hwnd, int dwTime, int dwFlags);

    }

    /// <summary>
    /// Methoden die aus der Kernel32.dll stammen
    /// </summary>
    public class Kernel32
    {
        /// <summary>
        /// The GlobalFindAtom function searches the global atom table 
        /// for the specified character string and retrieves the global 
        /// atom associated with that string.
        /// </summary>
        /// <param name="lpString"></param>
        /// <returns></returns>
        [DllImport("kernel32.dll", SetLastError = true, CharSet = CharSet.Auto)]
        public static extern ushort GlobalFindAtom(string lpString);


        /// <summary>
        /// The GlobalAddAtom function adds a character string to the global atom table
        /// and returns a unique value (an atom) identifying the string.
        /// </summary>
        /// <param name="lpString"></param>
        /// <returns></returns>
        [DllImport("kernel32", SetLastError = true)]
        public static extern ushort GlobalAddAtom(string lpString);


        /// <summary>
        /// The GlobalDeleteAtom function decrements the reference count of
        /// a global string atom. If the atom's reference count reaches zero,
        /// GlobalDeleteAtom removes the string associated with the atom from the
        /// global atom table.
        /// </summary>
        /// <param name="nAtom"></param>
        /// <returns></returns>
        [DllImport("kernel32", SetLastError = true)]
        public static extern ushort GlobalDeleteAtom(ushort nAtom);
    }
}
